import numpy as np
from sympy.physics.quantum.cg import CG


def jj_to_cubic_relativistic_harmonics(orbital='f'):
    if 'f' == orbital:
        jj_to_cubic = np.zeros((14,14))
        jj_to_cubic[0,8] = -np.sqrt(1./6.) # |5/2, -5/2>
        jj_to_cubic[4,8] =  np.sqrt(5./6.) # |5/2, +3/2> G7, 5/2, +
        jj_to_cubic[5,10] = -np.sqrt(1./6.) # |5/2, +5/2>
        jj_to_cubic[1,10] =  np.sqrt(5./6.) # |5/2, -3/2> G7, 5/2, -

        jj_to_cubic[4,0] =  np.sqrt(1./6.) # |5/2, +3/2>
        jj_to_cubic[0,0] =  np.sqrt(5./6.) # |5/2, -5/2> G81, 5/2, +
        jj_to_cubic[1,2] =  np.sqrt(1./6.) # |5/2, -3/2>
        jj_to_cubic[5,2] =  np.sqrt(5./6.) # |5/2, +5/2> G81, 5/2, -

        jj_to_cubic[3,4] =  1. # |5/2, +1/2> G82, 5/2, +
        jj_to_cubic[2,6] =  1. # |5/2, -1/2> G82, 5/2, -

        jj_to_cubic[13,12] =  np.sqrt(5./12.) # |7/2, +7/2>
        jj_to_cubic[ 9,12] =  np.sqrt(7./12.) # |7/2, -1/2> G6, 7/2, +
        jj_to_cubic[ 6,13] =  np.sqrt(5./12.) # |7/2, -7/2>
        jj_to_cubic[10,13] =  np.sqrt(7./12.) # |7/2, +1/2> G6, 7/2, -

        jj_to_cubic[12,11] = -np.sqrt(3./4.) # |7/2, +5/2>
        jj_to_cubic[ 8,11] =  np.sqrt(1./4.) # |7/2, -3/2> G7, 7/2, +
        jj_to_cubic[ 7,9] =  np.sqrt(3./4.) # |7/2, -5/2>
        jj_to_cubic[11,9] = -np.sqrt(1./4.) # |7/2, +3/2> G7, 7/2, -

        jj_to_cubic[13,7] =  np.sqrt(7./12.) # |7/2, +7/2>
        jj_to_cubic[ 9,7] = -np.sqrt(5./12.) # |7/2, -1/2> G81, 7/2, +
        jj_to_cubic[ 6,5] = -np.sqrt(7./12.) # |7/2, -7/2>
        jj_to_cubic[10,5] =  np.sqrt(5./12.) # |7/2, +1/2> G81, 7/2, -

        jj_to_cubic[12,3] = -np.sqrt(1./4.)  # |7/2, +5/2>
        jj_to_cubic[ 8,3] = -np.sqrt(3./4.)  # |7/2, -3/2> G82, 7/2, +
        jj_to_cubic[ 7,1] =  np.sqrt(1./4.)  # |7/2, -5/2>
        jj_to_cubic[11,1] =  np.sqrt(3./4.)  # |7/2, +3/2> G82, 7/2, -
    else:
        raise ValueError('UndefinedFunction')
    return jj_to_cubic


def comp_sph_harm_to_real_harm(dim_m):
    csh2rh = np.zeros((dim_m, dim_m), dtype=complex)
    # qunatum number l
    l = dim_m//2
    # set the orbital index
    iy = list(range(dim_m))
    for i in range(l):
        iy += [iy.pop(0)]
    for m in range(-l,l+1):
        if m < 0:
            csh2rh[iy[m], iy[m]] = 1.j/np.sqrt(2.)
            csh2rh[iy[-m], iy[m]] = -1.j/np.sqrt(2.)*(-1)**m
        elif m == 0:
            csh2rh[iy[m], iy[m]] = 1.
        else:
            csh2rh[iy[-m], iy[m]] = 1./np.sqrt(2)
            csh2rh[iy[m], iy[m]] = 1./np.sqrt(2)*(-1)**m
    return csh2rh


def get_u_csh2rh_all(ncorbs_list):
    u_csh2rh_list = [comp_sph_harm_to_real_harm(ncorbs) for \
            ncorbs in ncorbs_list]
    return u_csh2rh_list


def comp_sph_harm_to_relativistic_harm(dim_ms):
    '''transformation matrix from spin-complex spherical harmonics
    (orbital fast) to relativistic harmonics
    '''
    csh2relh = np.zeros((dim_ms, dim_ms), dtype=complex)
    dim_m = dim_ms//2
    l = dim_m//2
    # set the orbital index
    iy = list(range(dim_m))
    for i in range(l):
        iy += [iy.pop(0)]
    # add slow spin index
    # for spin: up, then dn. wien2k convention.
    iys = {0.5:iy, -0.5:[iy[i]+dim_m for i in range(dim_m)]}
    # j=l-1/2 block
    # relativistic_harmonics index
    i_jm = -1
    for i in [-0.5,0.5]:
        _j = l+i
        for mj in np.arange(-_j, _j+1):
            i_jm += 1
            for s in [-0.5,0.5]:
                csh2relh[iys[s][int(round(mj-s))], i_jm] = \
                        CG(l,mj-s,0.5,s,_j,mj).doit()
    return csh2relh


def get_u_csh2relh_all(ncorbs_list):
    u_csh2relh_list = [comp_sph_harm_to_relativistic_harm(ncorbs) for \
            ncorbs in ncorbs_list]
    return u_csh2relh_list


def get_u_csh2wan_all(ncorbs_list):
    ncorbs = ncorbs_list[0]
    if ncorbs%2 == 0: #with spin-orbit interaction
        u_csh2wan_list = get_u_csh2relh_all(ncorbs_list)
    else:
        u_csh2wan_list = get_u_csh2rh_all(ncorbs_list)
    return u_csh2wan_list



from __future__ import print_function
import h5py, numpy
from pygrisb.symm.angular_momentum_1p import get_L_vector_CH
from pygrisb.symm.atom_symm import get_representation, get_product_table
from pygrisb.math.matrix_util import sym_array


# d complex spherical Harmonics
L = get_L_vector_CH(2)

# get Lie parameters and one-particle dm to test
with h5py.File('data_complex_spherical_harmonics.h5', 'r') as f:
    nks = f['/NKS'][()].T
    lie_params = f['/lie_even_params'][()]

# take one-spin component
nks = nks[::2, ::2].T

print('nks.real in comp_sph_harm basis:')
for row in nks:
    print(('{:10.4f}'*len(row)).format(*row.real))

print('nks.imag in comp_sph_harm basis:')
for row in nks:
    print(('{:10.4f}'*len(row)).format(*row.imag))

# rotations
R_list = get_representation(L, lie_params)

# check product table
ptable = get_product_table(R_list)
print('ptable\n',ptable)

for i,rot in enumerate(R_list):
    err = nks.dot(rot) -  rot.dot(nks)
    print('{} maxerr of commutation = {}'.format(i,
            numpy.max(numpy.abs(err))))


nks = numpy.loadtxt('dm_ldau.txt').T

print('nks.real in comp_sph_harm basis:')
for row in nks:
    print(('{:10.4f}'*len(row)).format(*row.real))

print('nks.imag in comp_sph_harm basis:')
for row in nks:
    print(('{:10.4f}'*len(row)).format(*row.imag))

for i,rot in enumerate(R_list):
    err = nks.dot(rot) -  rot.dot(nks)
    print('{} maxerr of commutation = {}'.format(i,
            numpy.max(numpy.abs(err))))




import numpy as np

data_u1 = np.loadtxt('mu_u1.dat').T
data_u25 = np.loadtxt('mu_u2.5.dat').T


import matplotlib.pyplot as plt
plt.figure(1)
plt.plot(data_u1[0], data_u1[1], label='u=1.0')
plt.plot(data_u25[0], data_u25[1], label='u=2.5')
plt.legend()
plt.ylabel('$e_{tot}$')
plt.xlabel('$\mu$')
plt.figure(2)
plt.plot(data_u1[0], data_u1[2], label='u=1.0')
plt.plot(data_u25[0], data_u25[2], label='u=2.5')
plt.legend()
plt.ylabel('$n_f$')
plt.xlabel('$\mu$')
plt.show()



import numpy


def primitive_vec2_reciprocal_primvec(avec):
    '''
    given primitive vector, return reciprocal primitive vector.
    '''
    bvec = []
    ac = numpy.cross(avec[1], avec[2])
    bvec.append(numpy.pi*2*ac/(avec[0].dot(ac)))
    ac = numpy.cross(avec[2], avec[0])
    bvec.append(numpy.pi*2*ac/(avec[1].dot(ac)))
    ac = numpy.cross(avec[0], avec[1])
    bvec.append(numpy.pi*2*ac/(avec[2].dot(ac)))
    return numpy.array(bvec)


def get_k_label(kpt, br_latt):
    '''
    given the bravais lattice br_latt and the k-point in fractions
    of primitive reciprocal lattice vectors, return the k-label.
    Ref. W. Setyawan, S. Curtarolo, comp. mater. sci. 49, 299.
    '''
    if br_latt == 'op':
        if numpy.allclose(kpt, [0., 0., 0.]):
            return 'GAMMA'
        elif numpy.allclose(kpt, [0.5, 0., 0.5]):
            return 'U'
        elif numpy.allclose(kpt, [0.5, 0.5, 0.5]):
            return 'R'
        elif numpy.allclose(kpt, [0.5, 0., 0.]):
            return 'X'
        elif numpy.allclose(kpt, [0.5, 0.5, 0.0]):
            return 'S'
        elif numpy.allclose(kpt, [0., 0.5, 0.]):
            return 'Y'
        elif numpy.allclose(kpt, [0., 0.5, 0.5]):
            return 'T'
        elif numpy.allclose(kpt, [0., 0., 0.5]):
            return 'Z'
        else:
            return ' '
    else:
        # to be implemented.
        return ' '



import os, sys, h5py
import numpy as np
from pygrisb.model import semicir as bethlatt
import subprocess
from builtins import zip

import matplotlib
matplotlib.rcParams['backend'] = "Qt5Agg"
import matplotlib.pyplot as plt


def generate_data(u_list, mu_list):
    '''run *CyGutz* calculations for a list of U and mu.
    save the results in results.

    Parameters:

    * u_list: real array
      list of Hubbard U parameters.
    * mu_list: real array
      list of chemical potential

    Result:

    save the list of energies (e_list), double occupancy (d_list),
    quasi-particle weight (z_list) and occupation (n_list)
    to ``result.dat`` text file as well as the ``result.h5`` hdf5 file.
    '''
    # get *CyGutz* command. Choose option '-r -1' to skip the claculation of
    # Gutzwiller renormalized electron density.
    root = os.environ['WIEN_GUTZ_ROOT2']
    cmd = [root+'/CyGutz', '-r', '-1']

    # set chemical potential to 0.
    mu=0

    # set Hubbard U=0
    u = 0.

    # generate input files with updated with u=0, mu=0
    bethlatt.gutz_model_setup(u=u, nmesh=5000, mu=mu)

    # total energy list
    e_list = []

    # quasi-particle weight z list
    z_list = []

    # double occupancy d list
    d_list = []

    # occupation list
    n_list = []

    # loop over the list of U.
    for u, mu in zip(u_list, mu_list):

        print(' working on u = {} mu = {}'.format(u, mu))

        # modify the local Coulomb matrix
        with h5py.File('GPARAM.h5', 'a') as f:

            # note the transposition, which is transformation
            # from Fortran convention to c-convention.
            v2e = f['/IMPURITY_1/V2E'][()].T
            vdc2_list = f['VDC2_LIST'][()].T

            # now update the Coulom matrix
            v2e[0,0,0,0] = v2e[0,0,1,1] = v2e[1,1,0,0] = v2e[1,1,1,1] = u
            f['/IMPURITY_1/V2E'][()] = v2e.T

            # update vdc2, which keeps the particle-hole symmetry of the model
            # for finite U.
            vdc2_list[0, 0:2] = -u/2 + mu
            f['VDC2_LIST'][()] = vdc2_list.T

        # perform the *CyGutz* calculation.
        subprocess.call(cmd)

        # get total energy
        with h5py.File('GLOG.h5', 'r') as f:
            nks = f['/IMPURITY_1/NKS_SYM'][()]
            n = nks[0, 0] + nks[1, 1]
            n_list.append(n.real)
            e = f['/etot_model'][0] + u/2*n -mu*n
            e_list.append(e.real)

        # get Z = R^\dagger R
        with h5py.File('WH_RL_OUT.h5', 'r') as f:
            r = f['/IMPURITY_1/R'][0,0]

            # simple here since it is just a scalar
            z = r*r.conj()
            z_list.append(z.real)

        # to get double occupancy (of impurity 1), <n_up n_dn>_G,
        # we run analysis code *exe_spci_analysis*
        subprocess.call([root+'/exe_spci_analysis', '1'])

        # double occupancy is simply the local many-body density matrix element
        # in the valence=2 block.
        with h5py.File('EMBED_HAMIL_ANALYSIS_1.h5', 'r') as f:
            if '/valence_block_2/RHO' in f:
                d = f['/valence_block_2/RHO'][0, 0]
            else:
                d = 0.
            d_list.append(d.real)

    with open('result.dat', 'w') as f:
        for u, mu, e, z, d, n in zip(u_list, mu_list,
                e_list, z_list, d_list, n_list):
            f.write('{} {} {} {} {} {}\n'.format(u, mu, e, z, d, n))

    with h5py.File('result.h5', 'w') as f:
        f['/mu_list'] = mu_list
        f['/u_list'] = u_list
        f['/e_list'] = e_list
        f['/z_list'] = z_list
        f['/d_list'] = d_list
        f['/n_list'] = n_list


def scan_u(mu=0.0):
    '''run *CyGutz* calculations for a list of U for a given mu.

    Parameters:

    * mu: real number
      the fixed chemical potential.

    Result:

    it will generate results for a u_list of np.arange(0.0, 5.1, 0.2)
    at fixed mu.
    '''
    if os.path.isfile('result.h5'):
        return

    # set range of Hubbard U.
    u_list = np.arange(0.0, 5.1, 0.2)
    mu_list = [mu for u in u_list]
    generate_data(u_list, mu_list)


def scan_mu(u=5.0):
    '''run *CyGutz* calculations for a list of mu for a given u.

    Parameters:

    * u: real number
      the fixed Hubbard U

    Result:

    it will generate results for a mu_list = np.arange(0.0, 3.1, 0.1)
    at fixed u.
    '''
    if os.path.isfile('result.h5'):
        return

    # set range of chemical potential mu.
    mu_list = np.arange(0.0, 3.1, 0.1)
    u_list = [u for mu in mu_list]
    generate_data(u_list, mu_list)


def plot_scan_u():
    with h5py.File('result.h5', 'r') as f:
        u_list = f['/u_list'][()]
        e_list = f['/e_list'][()]
        z_list = f['/z_list'][()]
        d_list = f['/d_list'][()]

    f, axarr = plt.subplots(3, sharex=True)
    axarr[0].plot(u_list, e_list)
    axarr[0].set_ylabel('energy')
    axarr[1].plot(u_list, d_list)
    axarr[1].set_ylabel('double occupancy')
    axarr[2].plot(u_list, z_list)
    axarr[2].set_ylabel('Z')
    axarr[2].set_xlabel('U')
    axarr[2].set_xlim(min(u_list), max(u_list))
    plt.show()
    f.savefig('result.png')


def plot_scan_mu():
    with h5py.File('result.h5', 'r') as f:
        mu_list = f['/mu_list'][()]
        e_list = f['/e_list'][()]
        z_list = f['/z_list'][()]
        d_list = f['/d_list'][()]
        n_list = f['/n_list'][()]

    f, axarr = plt.subplots(4, sharex=True)
    axarr[0].plot(mu_list, e_list)
    axarr[0].set_ylabel('energy')
    axarr[1].plot(mu_list, d_list)
    axarr[1].set_ylabel('double occupancy')
    axarr[2].plot(mu_list, z_list)
    axarr[2].set_ylabel('Z')
    axarr[3].plot(mu_list, n_list)
    axarr[3].set_ylabel('n')
    axarr[3].set_xlabel('$\mu$')
    axarr[3].set_xlim(min(mu_list), max(mu_list))
    plt.show()
    f.savefig('result.png')



if __name__=='__main__':
    if '-mu' in sys.argv:
        scan_mu()
        plot_scan_mu()
    else:
        scan_u()
        plot_scan_u()



#!/usr/bin/env python

print """
Python library for preprocessing, managing and postprocessing
the (DFT+) Gutzwiller-RISB calculations.
It is used by [CyGutz](https://github.com/yaoyongxin/CyGutz)
and [comsuite](https://www.bnl.gov/comscope/software/comsuite.php).
"""



import h5py, glob, numpy
from pygrisb.estructure.fermi import get_fermi_level
from pygrisb.iface.ifwannier import get_risb_bndes


with h5py.File("GPARAMBANDS.h5", "r") as f:
    num_elec = f['/nelectron'][0]
    ismear = f["/ismear"][0]
    delta = f["/delta"][0]
print("num of electrons = {}".format(num_elec))

bnd_es = get_risb_bndes()
nk = bnd_es.shape[1]
wklist = [1./nk for k in range(nk)]

print(bnd_es[0][0])
with h5py.File("gband_es.h5", "w") as f:
    f["/gbnd_es"] = bnd_es

efermi = get_fermi_level(bnd_es, wklist, num_elec,
        delta=delta, ismear=ismear)
print("lda fermi level = {}".format(efermi))



from __future__ import print_function

import numpy as np
import unittest
import pygrisb.symm.angular_momentum_1p as am
from builtins import zip

J = am.get_J_vector([3], 'RH')
S = am.get_S_vector_JJ([3])
L = am.get_L_vector_JJ([3])

class KnowValues(unittest.TestCase):
    def test_BasisTransformation(self):
        for _J, _S, _L in zip(J, S, L):
            self.assertTrue(np.max(np.abs(_J-_S-_L))<1.e-6)


if __name__ == "__main__":
    print("Tests for J_CH_Transformation.")
    unittest.main()




import os, sys, subprocess, h5py, numpy, shlex
from pygrisb.gutz import embedh
from pygrisb.mbody.basis import h5add_fock_states


def create_ged_inp(imp, h1e, lambdac, daalpha, v2e, istep=0, rtol=1.e-10,
        nev=1, maxiter=1000):
    '''create/update the input file for the parallel ed solver.
    '''
    with h5py.File("GEDINP_{}.h5".format(imp), "a") as f:
        if v2e is not None:
            ind_list = numpy.where(numpy.abs(v2e) > rtol)
            val_list = v2e[ind_list]
            # in fortran convention, one-based
            f["/v2e/nnz"] = [len(val_list)]
            ind_list = numpy.array(ind_list)+1
            f["/v2e/INDS"] = ind_list.T
            # absorb the 1/2 factor.
            f["/v2e/vals"] = val_list/2.

        v1e = embedh.get_whole_h1e(h1e, lambdac, daalpha)
        ind_list = numpy.where(numpy.abs(v1e) > rtol)
        val_list = v1e[ind_list]
        if "/v1e" in f:
            del f["/v1e"]
        norb = v1e.shape[0]
        f["/v1e/norb"] = [norb]
        f["/v1e/nnz"] = [len(val_list)]
        ind_list = numpy.array(ind_list)+1
        f["/v1e/INDS"] = ind_list.T
        f["/v1e/vals"] = val_list

        if "/ed" not in f:
            f["/ed/nev"] = [nev]
            f["/ed/maxiter"] = [maxiter]
            f["/ed/rtol"] = [rtol]
            h5add_fock_states(norb/2, f)


def driver_ed(imp=1, istep=0, mpiexec="mpirun -np 2 "):
    '''dmrg solver based on ITensor.
    '''
    if "-i" in sys.argv:
        imp = int(sys.argv[sys.argv.index("-i")+1])
    if os.path.isfile("GEDINP_{}.h5".format(imp)):
        lv2e = False
    else:
        lv2e = True
    h1e, lambdac, daalpha, v2e = embedh.h5gen_embedh_spin_updn(
            imp=imp, lv2e=lv2e)
    create_ged_inp(imp, h1e, lambdac, daalpha, v2e, istep=istep)

    if os.path.isfile("mpi_ed.txt"):
        mpiexec = open("mpi_ed.txt", "r").readline()

    mpiexec = shlex.split(mpiexec)
    cmd = mpiexec+[os.environ["WIEN_GUTZ_ROOT2"]+"/exe_ed", "-i", str(imp)]

    print(" running {}".format(" ".join(cmd)))
    subprocess.call(cmd)

    # get the calculation results in cmp_sph_harm basis with
    # faster spin index.
    with h5py.File("GEDOUT_{}.h5".format(imp)) as f:
        dm = f["/DM"][()].T
        emol = f["/emol"][0]

    # transform it to the symmetry-adpated basis
    embedh.h5wrt_dm_sab_cmplx(dm, emol, imp=imp)



if __name__ == "__main__":
    driver_ed()



import numpy, h5py
from mpi4py import MPI
from scipy.io import FortranFile


def get_wannier_data_binary(fname):
    with FortranFile(fname, "r") as f:
        # real space lattice vectors
        reals_lat = f.read_reals().reshape((3, 3))
        # reciprocal space lattice vectors
        recip_lat = f.read_reals().reshape((3, 3))
        # maximal number of bands
        num_bands = f.read_ints()[0]
        # number of wannier orbitals
        num_wann = f.read_ints()[0]
        # k-point mesh
        ndiv = f.read_ints()
        # total number of k-points
        nqdiv = ndiv[0]*ndiv[1]*ndiv[2]
        # k-points
        kpts = f.read_reals().reshape((nqdiv, 3))
        # low energy band indices included in the wannier construction
        include_bands = f.read_ints()
        # list of overlap between band wavefunctions and wannier orbitals
        wfwannier_list = f.read_reals().view(numpy.complex).reshape(\
                (1, nqdiv, num_wann, num_bands)).swapaxes(2, 3)
        # list of band energies
        bnd_es = f.read_reals().reshape((1, nqdiv, num_bands))
    return reals_lat, recip_lat, kpts, include_bands, wfwannier_list, bnd_es


def h5get_wannier_data(fname):
    with h5py.File(fname, "r") as f:
        # real space lattice vectors
        reals_lat = f["/rbas"][()]
        # reciprocal space lattice vectors
        recip_lat = f["/gbas"][()]
        # maximal number of bands
        num_bands = f["/num_bands"][0]
        # number of wannier orbitals
        num_wann = f["/num_wann"][0]
        # k-point mesh
        ndiv = f["/ndiv"][()]
        # total number of k-points
        nqdiv = ndiv[0]*ndiv[1]*ndiv[2]
        # k-points
        kpts = f["/kpt_latt"][()]
        # low energy band indices included in the wannier construction
        include_bands = f["/include_bands"][()]
        # list of overlap between band wavefunctions and wannier orbitals
        wfwannier_list = f["/v_matrix"][()].reshape(\
                (1, nqdiv, num_wann, num_bands)).swapaxes(2, 3)
        # list of band energies
        bnd_es = f["/eigenvalues"][()].reshape((1, nqdiv, num_bands))
    return reals_lat, recip_lat, kpts, include_bands, wfwannier_list, bnd_es


def get_wannier_data(path="./"):
    fname = "{}/wannier.dat".format(path)
    try:
        f = h5py.File(fname, "r")
        f.close()
        h5input = True
    except:
        h5input = False
        pass
    if h5input:
        reals_lat, recip_lat, kpts, include_bands, wfwannier_list, bnd_es = \
                h5get_wannier_data(fname)
    else:
        reals_lat, recip_lat, kpts, include_bands, wfwannier_list, bnd_es = \
                get_wannier_data_binary(fname)
    return reals_lat, recip_lat, kpts, include_bands, wfwannier_list, bnd_es


def mpiget_wannier_data(path="./"):
    '''get the contents in wannier.dat Fortran binary file.
    '''
    comm = MPI.COMM_WORLD
    rank = comm.Get_rank()
    if rank == 0:
        reals_lat, recip_lat, kpts, include_bands, wfwannier_list, bnd_es =\
                get_wannier_data(path=path)
    else:
        reals_lat = recip_lat = kpts = include_bands = wfwannier_list = \
                bnd_es = None
    reals_lat = comm.bcast(reals_lat, root=0)
    recip_lat = comm.bcast(recip_lat, root=0)
    kpts = comm.bcast(kpts, root=0)
    include_bands = comm.bcast(include_bands, root=0)
    wfwannier_list = comm.bcast(wfwannier_list, root=0)
    bnd_es = comm.bcast(bnd_es, root=0)
    return reals_lat, recip_lat, kpts, include_bands, wfwannier_list, bnd_es


if __name__ == "__main__":
    get_wannier_data()



import numpy as np
from pygrisb.model import circauxi
import shutil,subprocess

mu_list = np.arange(0.0,0.1,0.1)
cmd = ['/home/ykent/WIEN_GUTZ/bin2/CyGutz', '-r', '-1']
for mu in mu_list:
    circauxi.gutz_model_setup(u=2.5, nmesh=5000, norb=3, tiny=0.0, mu=mu)
    subprocess.call(cmd)
    shutil.copyfile('WH_RL_BEST.h5', 'WH_RL_INIT.h5')
    shutil.copyfile('WH_RL_BEST.h5', 'WH_RL_INIT.h5_u2.5mu{}'.format(mu))




import numpy as np
import h5py
import pygrisb.basic.units as units
import pygrisb.basic.splot as splot

'''
Equation of state.
'''


def Murnaghan(parameters, vol):
    '''
    Given a vector of parameters and volumes, return a vector of energies.
    equation From PRB 28,5480 (1983)
    '''
    E0 = parameters[0]
    B0 = parameters[1]
    BP = parameters[2]
    V0 = parameters[3]
    return E0 + B0 * vol / BP * (((V0 / vol)**BP) /  \
            (BP - 1) + 1) - V0 * B0 / (BP - 1.0)


def Murnaghan_pv(parameters, vol):
    '''
    function P(V).
    '''
    B0 = parameters[1]
    BP = parameters[2]
    V0 = parameters[3]
    return B0 / BP * ((V0 / vol)**BP - 1.0)


def eos_fit_fun(pars, y, x):
    '''
    The objective function that will be minimized.
    '''
    return y - Murnaghan(pars, x)


def get_ev_fit(v, e):
    '''
    Fitting the Birch-Murnaghan EOS to data. v in \A^3, e in eV.
    Based on http://gilgamesh.cheme.cmu.edu/doc/software/jacapo/
    appendices/appendix-eos.html
    '''
    from pylab import polyfit
    from scipy.optimize import leastsq

    # fit a parabola to the data
    # y = ax^2 + bx + c
    a, b, c = polyfit(v, e, 2)
    '''The parabola does not fit the data very well, but we can use it to get
    some analytical guesses for other parameters.
    V0 = minimum energy volume, or where dE/dV=0
    E = aV^2 + bV + c
    dE/dV = 2aV + b = 0
    V0 = -b/2a
    E0 is the minimum energy, which is:
    E0 = aV0^2 + bV0 + c
    B is equal to V0*d^2E/dV^2, which is just 2a*V0
    and from experience we know Bprime_0 is usually a small number like 4
    '''
    # now here are our initial guesses.
    v0 = -b / (2 * a)
    e0 = a * v0**2 + b * v0 + c
    b0 = 2 * a * v0
    bP = 4
    # initial guesses in the same order used in the Murnaghan function
    x0 = [e0, b0, bP, v0]
    murnpars, ier = leastsq(eos_fit_fun, x0, args=(e, v))
    return murnpars


def h5get_mfit_ev(nmesh_fac=10, fsave='results.h5', path='/lapw'):
    '''Calculate and save Murnaghan fiting results in fsave.
    Interpolated e-v and p-v data on volume mesh with a factor a
    nmesh_fac of the original one are also stored.
    '''
    # Get e,v data.
    with h5py.File(fsave, 'r') as f:
        e_list = f[path+'/etot_list'][...]
        v_list = f['/vol_list'][...]

    # fitting
    murnpars = get_ev_fit(v_list, e_list)

    vh = np.linspace(v_list[0], v_list[-1], nmesh_fac * len(v_list) - 1)
    eh = Murnaghan(murnpars, vh)
    ph = Murnaghan_pv(murnpars, vh)*units.eVA_GPa
    with h5py.File(fsave, 'a') as f:
        if path+'/eosfit' in f:
            del f[path+'/eosfit']
        f[path+'/eosfit/e0'] = murnpars[0]
        f[path+'/eosfit/b0'] = murnpars[1]
        f[path+'/eosfit/bp'] = murnpars[2]
        f[path+'/eosfit/v0'] = murnpars[3]
        f[path+'/eosfit/v_list'] = vh
        f[path+'/eosfit/e_list'] = eh
        f[path+'/eosfit/p_list'] = ph
    splot.xy2_plot([v_list, vh], [e_list, eh], ['o', '-'], ['raw', 'fitting'],
            xlabel='V ($\AA^3$/primitive cell)',
            ylabel='E (eV/primitive cell)', fsave=path+'_evfit.pdf')
    splot.xy_plot(vh, ph, xlabel='V ($\AA^3$/primitive cell)',
            ylabel='P (GPa)', fsave=path+'_pvfit.pdf')


def eos_spline(v, e, tol):
    '''
    Get volume, energy, pressure, and bulk modulus using spline, given
    v in \A^3 and e in eV.
    '''
    from scipy.interpolate import UnivariateSpline
    s = UnivariateSpline(v, e, k=3, s=tol)
    vh = np.linspace(v[0], v[-1], 10 * len(v) - 1)
    eh = [s.derivatives(i)[0] for i in vh]
    ph = [-s.derivatives(i)[1] * units.eVA_GPa for i in vh]
    bh = [s.derivatives(i)[2] * vh[i] * units.eVA_GPa for i in vh]
    return vh, eh, ph, bh



#!/usr/bin/env python

import h5py
import numpy as np
from pygrisb.math.matrix_basis import sigmatomatrixbasis, matrixstructtobasis
from pygrisb.gutz.usrqa import get_usr_input

def get_shrink_sigma(sigma):
    '''
    Shrink the structure of sigma if numbers skipped.
    '''
    elem_max = np.max(sigma)
    for elem in range(elem_max):
        if elem == np.max(sigma):
            break
        while elem + 1 not in sigma:
            spots = np.where(sigma > elem)
            sigma[spots] -= 1
    return sigma


def print_sigma(sigma, title):
    '''
    Print the structure of sigma with basis indices.
    '''
    print(title)
    print("index  " + ''.join("%4d " % (i) for i in range(len(sigma))) + '\n')
    for i, row in enumerate(sigma):
        print("%4d   " % (i) + ''.join("%4d " % (j) for j in row))


def init_mott(fin='GPARAM.h5', fout='GMOTT.h5'):
    '''
    Generate the input files for Mott phase calculation.
    '''
    dim_hs_r_list = []
    dim_hs_l_list = []
    f = h5py.File(fin, 'r')
    fp = h5py.File(fout, 'w')
    num_imp = f["/num_imp"][...]
    for imp in range(num_imp):
        prepath = "/IMPURITY_" + str(imp+1)
        sigma = f[prepath + "/SIGMA_STRUCT"][...]
        print("**********  Impurity " + str(imp+1) + "  **********")
        print_sigma(sigma, ' Sigma structure:')
        while True:
            orb_mott = input(
                    ' Please provide the indices of orbitals to'
                    +' be Mott localized \n (e.g., 0 2 ): ')
            yn = input(
                ' You selected [' +
                orb_mott +
                '] \n to be Mott localized, right? (y/n):')
            if 'y' in yn or 'Y' in yn:
                orb_mott = orb_mott.split()
                ind_orb_mott = [int(s) for s in orb_mott]
                while True:
                    ne_mott = input(
                            ' Please provide the total number of'
                            +' Mott localized electrons (per unit cell): ')
                    yn = input(
                        ' Total ' +
                        ne_mott +
                        ' electrons will be Mott localized, right? (y/n):')
                    if 'y' in yn or 'Y' in yn:
                        break
                break
        fp[prepath+'/num_mott_orbitals'] = [len(ind_orb_mott)]
        fp[prepath+'/num_mott_electrons'] = [int(ne_mott)]
        fp[prepath+'/mott_orbital_indices'] = np.asarray(ind_orb_mott)+1

        for ind in ind_orb_mott:
            sigma[ind, :] = 0
        sigma = get_shrink_sigma(sigma)
        print_sigma(sigma, ' R structure:')
        hs = matrixstructtobasis(sigma)
        dim_hs_r_list.append(len(hs))

        fp[prepath+'/SIGMA_STRUCT_R'] = sigma.T
        if len(hs) > 0:
            fp[prepath+'/HS_R'] = np.swapaxes(hs,1,2)

        for ind in ind_orb_mott:
            sigma[:, ind] = 0
        sigma = get_shrink_sigma(sigma)
        print_sigma(sigma, ' Lambda structure:')
        hs = sigmatomatrixbasis(sigma)
        dim_hs_l_list.append(len(hs))
        fp[prepath+'/SIGMA_STRUCT_L'] = sigma.T
        if len(hs) > 0:
            fp[prepath+'/HS_L'] = np.swapaxes(hs,1,2)

    print("\n Please select the method to solve embedding Hamiltonian.\n" +
            " LDIAG = -2: Valence truncation ED for Mott solution.\n" +
            "         -3: Option (-2) with additional S=0 constraints.\n" +
            "         -4: Option (-2) with additional J=0 constraints.\n" +
            "        -31: Option (-2) with S=0 \n" +
            "             and local symmetry constraints.")
    iembeddiag = get_usr_input(" Please select LDIAG: ",
            ['-2','-3','-4','-31'])

    fp['/giembeddiag'] = [int(iembeddiag)]
    fp['/dim_hs_r_imp'] = dim_hs_r_list
    fp['/dim_hs_l_imp'] = dim_hs_l_list
    f.close()
    fp.close()


if __name__ == "__main__":
    init_mott()



import numpy as np
import pickle
from ase.dft import kpoints
import pygrisb.gutz.ginput as ginput
import pygrisb.model.tbASE as tb

def gutz_model_setup(u=0.0, spindeg=True, num_e=2., iembeddiag=-1):
    '''Set up Gutzwiller calculations for 2d body-centered square lattice.

    Parameters:

    * u: real number
      Hubbard U.
    * spindeg: boolean number
      whether to keep spin degeneracy or not.
    * num_e: real number
      number of electron per unit cell
    * iembeddiag: integer
      flag for method to solve the embedding Hamiltonian.

      * -3: valence truncation ED with S=0 (spin-singlet) constraint;
      * -1: valence truncation ED;
      * 10: Hartree-Fock.

    Result:

    Create all the necessary input file of ``GPARAMBANDS.h5``, ``GPARAM.h5``,
    and ``BAREHAM_0.h5``, for *CyGutz* calculation.
    '''

    # two "H" atoms in the square unit cell, one at the corner
    # and one at the center.
    symbols=['H', 'H']
    scaled_positions=[(0, 0, 0), (0.5, 0.5, 0)]
    cell = np.identity(3)
    a = tb.AtomsTB(symbols=symbols, scaled_positions=scaled_positions,
            cell=cell)

    # set spin degeneracy accordingly.
    a.set_orbitals_spindeg(spindeg=spindeg)

    # create a tight-binding model class given the AtomsTB.
    aTB = tb.TB(a)

    # set real space (nearest neighbour) hopping elements.
    t = -1.0
    aTB.set_hop([
            (( 0, 0,0),0,1, t),
            ((-1, 0,0),0,1, t),
            (( 0,-1,0),0,1, t),
            ((-1,-1,0),0,1, t),
            (( 0, 0,0),1,0, t),
            (( 1, 0,0),1,0, t),
            (( 0, 1,0),1,0, t),
            (( 1, 1,0),1,0, t),
            ])

    # set 2d k-mesh
    kps_size = (50, 50, 1)
    kps = kpoints.monkhorst_pack(kps_size)

    # set uniform k-point weight
    num_k = len(kps)
    kps_wt = 1.0 / num_k * np.ones((num_k))
    if aTB.Atoms.spindeg:
        kps_wt *= 2

    # se maximal number of bands (here we have two bands.)
    num_band_max = 2

    # set list of one-body part of the local Hamiltonian (trivial here.)
    h1e_list = [np.array([[0,]], dtype=np.complex) for symbol in symbols]

    # create ``GPARAMBANDS.h5`` file
    ginput.save_gparambands(kps_wt, num_e, num_band_max, h1e_list=h1e_list)

    # cerate ``BAREHAM_0.h5`` file.
    aTB.save_bareham(kps)

    # for initializing CyGutz calculation
    from pygrisb.gutz.batch_init import batch_initialize

    if spindeg:
        spin_polarization = 'n'
    else:
        spin_polarization = 'y'

    # the default settings are good for this model.
    batch_initialize(cell=cell, scaled_positions=scaled_positions,
            symbols=symbols,idx_equivalent_atoms=[0,1],
            unique_u_list_ev=[u], iembeddiag=iembeddiag,
            spin_polarization=spin_polarization,
            updn_full_list=[1,-1])

    # save the aTB
    with open('aTB.pckl', 'wb') as f:
        pickle.dump([a, aTB], f)


def get_bard_bands():
    '''get bare band structure.
    '''
    # load aTB
    with open('aTB.pckl', 'rb') as f:
        a, aTB = pickle.load(f)

    # k-point path
    kG = [0.0, 0.0, 0]
    kX = [0.5, 0.0, 0]
    kM = [0.5, 0.5, 0]

    # set up a ase.dft.kpoints kpath object
    kps = kpoints.get_bandpath([kG, kX, kM, kG], a.cell)

    # get band structure of a square lattice
    aTB.get_bandstructure(kps, saveto="bare_bands.dat")


if __name__=='__main__':
    gutz_model_setup()
    get_bard_bands()



from __future__ import print_function
import numpy
from builtins import zip
from scipy.special import erfc
from scipy.optimize import bisect


def get_fermi_level(bnd_es, wklist, num_e, delta=0.0258, \
        ismear=0, iso=1):
    # delta = 0.0258519909eV = 300K
    emin = numpy.min(bnd_es)
    emax = numpy.max(bnd_es)
    efermi = bisect(err_fun, emin, emax, \
            args=(bnd_es, wklist, num_e, delta, ismear, iso))
    return efermi


def err_fun(mu, bnd_es, wklist, num_e, delta, ismear, iso):
    ferwes = get_fermi_weight(mu, bnd_es, wklist, delta, ismear=ismear,
            iso=iso)
    _nume = numpy.sum(ferwes)
    return _nume - num_e


def get_fermi_weight(mu, bnd_es, wklist, delta=0.0258, \
        ismear=0, iso=1):
    ferwes = []
    for bnd_e in bnd_es:
        ferwes.append([])
        for bnd_ek, wk in zip(bnd_e, wklist):
            ferwes[-1].append([])
            for e in bnd_ek:
                x = (e-mu)/delta
                if ismear == -1:
                    fw = fermi_dirac(x)
                elif ismear == 0:
                    fw = gaussian(x)
                else:
                    raise ValueError("Not defined ismear = {}!".format(ismear))
                fw *= wk
                ferwes[-1][-1].append(fw)
    ferwes = numpy.asarray(ferwes)
    if len(ferwes) == 1 and iso == 1:
        ferwes *= 2
    return ferwes


def fermi_dirac(x):
    """
    Return fermi-dirac distribution weight.
    """
    if x < -200:
        f = 1.
    elif x > 200:
        f = 0.
    else:
        f = 1./(numpy.exp(x) + 1)
    return f


def gaussian(x):
    """
    Return gaussian distribution weight.
    """
    if x < -7:
        f = 2.
    elif x > 7:
        f = 0.
    else:
        f = erfc(x)
    return f/2.0



from __future__ import print_function

# generate the j_square basis for each valence block f-shell.
from pygrisb.mbody.local_operator_factory import get_nij_op, get_am_op_from_nij
from pygrisb.symm.angular_momentum_1p import get_J_vector
from pygrisb.symm.unitary import comp_sph_harm_to_relativistic_harm
import numpy, h5py, sys


def h5generate_j2_basis(l="f"):
    '''generate j2 eigen-states with relativistic harmonics as the
    single-particle basis.
    '''
    norbs = {"d":10, "f":14}
    l_list = [(norbs[l]/2-1)/2]
    jvec = get_J_vector(l_list)
    ucsh2rh = comp_sph_harm_to_relativistic_harm(norbs[l])
    jvec = [ucsh2rh.T.conj().dot(jcomp).dot(ucsh2rh) for jcomp in jvec]
    with h5py.File("j2evec.h5", "a") as f:
        group = "/{}".format(l)
        if group in f:
            del f[group]
        for val in range(1,norbs[l]):
            nij = get_nij_op(l=l, ival=val)
            jop_list = get_am_op_from_nij(jvec=jvec, nij=nij,
                    op_list=['Jz', 'J2'])
            j2op = jop_list["J2"].todense()
            jzop = jop_list["Jz"].todense()
            w,v = numpy.linalg.eigh(j2op)
            j_list = map(lambda x: abs(round(numpy.sqrt(x+0.25)-0.5, 1)), w)
            j_unique = numpy.unique(j_list)
            for j in j_unique:
                istr = j_list.index(j)
                iend = len(j_list) - j_list[::-1].index(j)
                v_j = v[:,istr:iend]
                jzop_j = v_j.T.conj().dot(jzop).dot(v_j)
                _w,_v = numpy.linalg.eigh(jzop_j)
                v_j = v_j.dot(_v)
                f["{}/val_{}/j_{}/v".format(group,val,j)] = v_j


def h5calc_dominant_config(l="f", fj2basis="j2evec.h5", \
        frho="EMBED_HAMIL_ANALYSIS_1.h5", fresult="mbbasis.h5", \
        rcut=1.e-3, entry="RHO"):
    if "-e" in sys.argv:
        entry = sys.argv[sys.argv.index("-e")+1]
    if "-f" in sys.argv:
        frho = sys.argv[sys.argv.index("-f")+1]

    norbs = {"d":10, "f":14}
    j_ranges = {0:numpy.arange(15), 1:numpy.arange(0.5,15.5)}
    j_list = []
    with h5py.File(fj2basis, "r") as fj:
        with h5py.File(frho, "r") as fin:
            with h5py.File(fresult, "a") as fout:
                for val in range(norbs[l]+1):
                    evals = []
                    evecs = None
                    path_data = "/valence_block_{}/{}".format(val, \
                            entry.upper())
                    if path_data in fin:
                        # fortran to c convention
                        rho = fin[path_data][()].T
                        evals = []
                        if rho.shape[0] == 1:
                            if rho[0,0] > rcut:
                                evals.append(rho[0,0])
                                evecs = [1.0+0.j]
                                j_list.append(0.0)
                        else:
                            for j in j_ranges[val%2]:
                                path_data = "/{}/val_{}/j_{:.1f}/v". \
                                        format(l,val,j)
                                if path_data in fj:
                                    u = fj[path_data][()]
                                    rho_j = -u.T.conj().dot(rho).dot(u)
                                    w,v = numpy.linalg.eigh(rho_j)
                                    w *= -1.
                                    for iw,w1 in enumerate(w):
                                        if w1 < rcut:
                                            if iw == 0 or \
                                                    w[iw-1]-w1 > 1.e-6:
                                                break
                                    if iw < len(w)-1:
                                        iw -= 1
                                    if iw >= 0:
                                        w = w[:iw+1]
                                        v = v[:,:iw+1]
                                        evals.extend(w)
                                        if evecs is None:
                                            evecs = u.dot(v)
                                        else:
                                            evecs = numpy.concatenate(\
                                                    (evecs, u.dot(v)), \
                                                    axis=1)
                                        if (iw+1) % int(round((2*j+1))) != 0:
                                            print("val = {} j = {}".format(\
                                                    val,j))
                                            print(w)
                                            raise ValueError(\
                                                    (" error: j = {} while "+\
                                                    "nevec = {}!").format(\
                                                    j,iw+1))
                                        j_list.extend([j for w1 in w])
                        if evecs is not None:
                            path = "/valence_block_{}".format(val)
                            fout[path+"/{}_EVEC".format(\
                                    entry.upper())] = evecs.T
                            fout[path+"/{}_eval".format(\
                                    entry.lower())] = evals
                            if path+"/nv" not in fout:
                                fout[path+"/nv"] = len(evals)
                if len(j_list) > 0:
                    fout["/{}_j_list".format(entry.lower())] = j_list



if __name__ == "__main__":
    # h5generate_j2_basis(l="f")
    h5calc_dominant_config()



from __future__ import print_function


import h5py
from pygrisb.io.h5io import get_csr_matrix
from pygrisb.symm.atom_symm import get_atom_U_sigma_chi


def get_U_chi_sigma(imp, valences):
    '''
    Get the U_list, chi_list and sigma_list in the Hilbert space.
    '''
    f = h5py.File("hilbert_space_rotations.h5", 'r')
    list_U = []
    list_chi = []
    list_sigma = []
    for val in valences:
        Rpr = []
        for j in range(f["Impurity_{}/val_block={}/dim_rotations".format( \
                imp, val)][...]):
            Rpr.append(get_csr_matrix(f, \
                    "Impurity_{}/val_block={}/rotation_{}".format( \
                    imp, val, j)).todense())
        U, sigma, chi = get_atom_U_sigma_chi(Rpr)
        list_U.append(U)
        list_chi.append(chi)
        list_sigma.append(sigma)
    f.close()
    return list_U, list_chi, list_sigma


if __name__ == "__main__":
    list_U, list_chi, list_sigma = get_U_chi_sigma(1, [2])
    print(list_sigma[0])
    for chi in list_chi[0]:
        print(''.join(" {:<4.1f}".format(x) for x in chi))
    print(list_U)



import numpy as np
from pygrisb.model import circauxi
import shutil,subprocess


cmd = ['/home/ykent/WIEN_GUTZ/bin2/CyGutz', '-r', '-1']
for i,u in enumerate(np.arange(1.0,0.9,-10)):
    print(' Running with u = {}'.format(u))
    circauxi.gutz_model_setup(u=u, nmesh=5000, norb=3, tiny=0.0, mu=0.0)
    subprocess.call(cmd)
    shutil.copyfile('WH_RL_BEST.h5', 'WH_RL_INIT.h5')
    shutil.copyfile('WH_RL_BEST.h5', 'WH_RL_INIT.h5_{}'.format(u))




from scan_checkboard import get_scan_data
import matplotlib.pyplot as plt

def plot_scan_u_pmafm():
    '''plot a figure, which compares the PM and AFM results from Gutzwiller
    calculations.
    '''
    # Get data from precalcuated PM results.
    u_list_pm, e_list_pm, z_list_pm, d_list_pm, m_list_pm = \
            get_scan_data(fname='result_pm')

    # Get data from precalcuated AFM results.
    u_list_afm, e_list_afm, z_list_afm, d_list_afm, m_list_afm = \
            get_scan_data(fname='result_afm')

    f, axarr = plt.subplots(2, 2, sharex=True)
    axarr[0, 0].plot(u_list_pm, e_list_pm, label='PM')
    axarr[0, 0].plot(u_list_afm, e_list_afm, label='AFM')
    axarr[0, 0].legend()
    axarr[0, 0].set_ylabel('energy')
    axarr[1, 0].plot(u_list_pm, d_list_pm)
    axarr[1, 0].plot(u_list_afm, d_list_afm)
    axarr[1, 0].set_ylabel('double occupancy')
    axarr[0, 1].plot(u_list_pm, z_list_pm)
    axarr[0, 1].plot(u_list_afm, z_list_afm)
    axarr[0, 1].yaxis.tick_right()
    axarr[0, 1].yaxis.set_label_position("right")
    axarr[0, 1].set_ylabel('Z')
    axarr[1, 1].plot(u_list_pm, m_list_pm)
    axarr[1, 1].plot(u_list_afm, m_list_afm)
    axarr[1, 1].set_ylabel('local $<S_{z}>$')
    axarr[1, 1].set_ylim(-1,1)
    axarr[1, 1].yaxis.tick_right()
    axarr[1, 1].yaxis.set_label_position("right")
    axarr[1, 0].set_xlabel('U')
    axarr[1, 1].set_xlabel('U')
    axarr[1, 0].set_xlim(min(u_list_pm), max(u_list_pm))
    plt.tight_layout()
    plt.show()
    f.savefig('result_pmafm.png')



if __name__=="__main__":
    plot_scan_u_pmafm()



import h5py, numpy
from mpi4py import MPI
from pygrisb.estructure.fermi import get_fermi_level
from pygrisb.estructure.gwannier import get_gmodel, mpiget_bndev
from pygrisb.iface.ifwannier import get_risb_bndes


with h5py.File("GPARAMBANDS.h5", "r") as f:
    num_elec = f['/nelectron'][0]
    ismear = f["/ismear"][0]
    delta = f["/delta"][0]
    kpts = f["/kpoints"][()]

comm = MPI.COMM_WORLD
rank = comm.Get_rank()
if rank == 0:
    print("num of electrons = {}".format(num_elec))

gmodel = get_gmodel()
#kpts = gmodel.k_uniform_mesh((6, 6, 6))
nk = len(kpts)
wklist = [1./nk for k in kpts]
bnd_es, bnd_vs = mpiget_bndev(kpts, gmodel, mode="risb")
if rank == 0:
    bnd_es2 = get_risb_bndes()
    for ik in range(nk):
        if numpy.max(numpy.abs(bnd_es2[0][ik] - bnd_es[0][ik])) > 1.e-6:
            print(ik)
            print(bnd_es2[0][ik])
            print(bnd_es[0][ik])
    efermi = get_fermi_level(bnd_es, wklist, num_elec,
            delta=delta, ismear=ismear)
    print("lda fermi level = {}".format(efermi))



from pygrisb.iface.ifwannier import get_wannier_dat


reals_latt, recip_latt, kpts, include_bands, wfwannier_list, bnd_es = \
        get_wannier_dat()

print(reals_latt)
print(recip_latt)

print(reals_latt.dot(recip_latt.T))



# Author: Yongxin Yao, Xiaoyu Deng

'''
A tight-binding model is defined by a set of hoppoing matrix H(R).
H(R) is a matrix with dimension of orbitals in the unit cell.
Dimension: all cells are three dimension in ASE,
however one can mimick two dimensional model
by setting hopping along the third axis zero.
'''

import numpy
import h5py
from ase import Atoms
from ase.dft import kpoints


class AtomsTB(Atoms):

    """
    ASE ATOMS object with orbitals.

    Attributes:

      - For attributes of ASE ATOMS object,
        please check ASE documentation.
      - orbitals: orbital names/index,
        list with each item is a list of orbital names
        on each atom, for example, [["s"],["s","p"]]
      - spin: name of spins, by default, spin=["up"],
        only one spin component is considered.
        Can be also ["up","dn"].
      - nspinorbitals: number of orbitals with possible
        spin degeneracy
    """

    def set_orbitals_spindeg(self, orbitals=None,
            spindeg=True, spinorbit=False):
        """
        Set the orbitals in a ASE Atoms object.

        Parameter:
        ----------
          orbitals: list
            orbitals names, list with each item
            is a list of orbital names on each atom,
            for example, [("s"),("s","px")]
          spindeg: bool
            spin degeneracy, if TRUE spin=["up"];
            if False, spin=["up","dn"]
          spinorbit: bool
            True, orbitals are in fact spin-orbitals,
            if this is true, spin is set to ["so"],
            False, orbitals are without spinorbit coupling.

        Example::

        >>> # set a atom with two bands, "s" and "p",
        >>> # and spin degenaracy
        >>> a=AtomsTB("N",[(0,0,0)],cell=(1,1,1))
        >>> a.set_orbitals_spindeg(orbitals=[("s","px")],
        >>>         spindeg=True)
        """

        if orbitals is None:
            # one s orbital per atoms by default.
            self.orbitals = [("s",) for pos in self.positions]
        else:
            self.orbitals = orbitals[:]

        self.spindeg = spindeg
        self.spinorbit = spinorbit
        if spinorbit:
            self.nso = 2
        else:
            self.nso = 1

        self.nspinorbitals = 0
        for i in self.orbitals:
            self.nspinorbitals += len(i)
        self.nspinorbitals *= self.nso


class TB(object):

    """
    A tight-binding mode is defined with an ASE object Atoms
    and Hopping.
    """

    def __init__(self, AtomsTB, Hr=None, hk_list=None):
        """
        Init TB object

        Parameters
        ----------
        AtomsTB: ASE object
          define the unit cell, the atoms, with orbitals set.
        Hr: dict
          a dict of (R, hop), with R is tranlational vector
          and hop is hopping matrix between unit cells.
          len(Hr[R])=self.AtomsTB.nspinorbitals.
        """

        self.Atoms = AtomsTB
        self.Hr = Hr
        self.hk_list = hk_list


    @staticmethod
    def gallery(name="SimpleSquareLattice"):
        """
        A gallery of predefined lattice that frequentely used.

        Parameters
        ----------
        name: model name
          currently "SimpleSquareLattice", "Chain_nn" implemented.

        Returns
        -------
        aTB: TB object
        """

        if name == "SimpleSquareLattice":
            a = AtomsTB("N", [(0, 0, 0)], cell=(1, 1, 1))
            a.set_orbitals_spindeg()
            aTB = TB(a)
            aTB.set_hop([((0, 1, 0), 0, 0, -1),
                         ((1, 0, 0), 0, 0, -1),
                         ((0, -1, 0), 0, 0, -1),
                         ((-1, 0, 0), 0, 0, -1)])
            return aTB

        if name == "Chain_nn":
            a = AtomsTB("N", [(0, 0, 0)], cell=(1, 1, 1))
            a.set_orbitals_spindeg()
            aTB = TB(a)
            aTB.set_hop([((1, 0, 0), 0, 0, 1),
                         ((-1, 0, 0), 0, 0, 1)])
            return aTB

    def set_hop(self, hoppings=None):
        """
        Set hopping mannually. In addition to a Hr matrix.
        Hopping is a tupe with the form (R,iorb,jorb,t_hop).
        hoppings could be a list of hoppings.

        Parameters
        ----------
        hoppings: list
          a list of hopping defined as tuple(R,iorb,jorb,t_hop).
          For example, [((0,0,0),0,0,1)], or a tuple(R,hr),
          where hr is a matrix of hoppings

        Example::

        >>>a=AtomsTB("N",[(0,0,0)],cell=(1,1,1))
        >>>a.set_orbitals_spindeg()
        >>>a.set_hop([((0,1,0),0,0,-1),
        >>>           ((1,0,0),0,0,-1)])
        """
        # if only set only one term, change it to list for consistency.
        if type(hoppings) == type((1,)):
            hoplist = [hoppings, ]
        else:
            hoplist = hoppings
        if self.Hr is None:
            self.Hr = {}
        for ihop in hoplist:             #
            if len(ihop) > 2:
                R, iorb, jorb, t_hop = ihop
                if R not in self.Hr:  # the matrix for R is not set yet
                    self.Hr[R] = numpy.zeros((
                            self.Atoms.nspinorbitals,
                            self.Atoms.nspinorbitals), dtype=numpy.complex)
                self.Hr[R][iorb, jorb] = t_hop
            else:
                R, hr = ihop
                self.Hr[R] = hr


    def Hk(self, ikp=0, kpt=numpy.zeros((3))):
        """
        Construct Hamiltonian for a given k point, i.e.,
        fourier transformation of Hr.

        Parameters
        ----------
        kpt: array
          size (3). k point is required to be in unit of reciprocal
          periodic lattice vector.

        Returns
        -------
        Hk: ndarray
          dimension (norbitals,norbitals), with each element
          the Hamiltonian for the corresponding kpoint.
        kcart: ndarray
          k points in cartesian coordinates.
        """

        if self.Hr is not None:
            reciprocalvector = self.Atoms.get_reciprocal_cell()
            self.hk = numpy.zeros((self.Atoms.nspinorbitals, \
                    self.Atoms.nspinorbitals), dtype=numpy.complex)
            ikcart = numpy.dot(kpt, reciprocalvector)
            for ir in self.Hr:
                R = numpy.array(ir)
                Rcart = numpy.dot(R, numpy.array(self.Atoms.cell))
                expk = numpy.exp(-1j * ikcart.dot(Rcart) * 2.0 * numpy.pi)
                self.hk += expk * self.Hr[ir]
        elif self.hk_list is not None:
            self.hk = self.hk_list[ikp]
            ikcart = None
        else:
            raise ValueError('Neither Hr nor hk_list are defined!')

        return self.hk, ikcart

    def eigens(self, ikp=0, kpt=numpy.zeros((3))):
        """
        Caculate eigenvalues of a given list of kpoints.
        It calls Hk and diagonalize the hamiltonian.

        Parameters
        ----------
        kpt: numpy.ndarry
          kpoint, dimension (3), in unit of reciprocal lattice vectors.

        Returns
        -------
        eks: ndarray
          eigenvalues, taken to be real.
        Uk: ndarray
          wavefunctions.
        """

        hk, _ = self.Hk(ikp, kpt)
        if hk.shape[0] == 1:
            ek, Uk = [hk[0,0].real], numpy.array([[1.0+0.j]])
        else:
            ek, Uk = numpy.linalg.eigh(hk)
        return ek, Uk

    def get_bandstructure(self, kps, saveto=None, with_weights=False):
        """
        Get the band structure for given kpath.

        Parameters
        ----------
        kps: list
          kpoints, kpath object given by ase.dft.kpoints.
        saveto: string
          name of the file to save data.
        with_weights:

        """

        eks = []
        if with_weights:
            Uks = []
        for i, kpt in enumerate(kps[0]):
            ek, Uk = self.eigens(i, kpt)
            eks.append(ek)
            if with_weights:
                Uks.append(Uk)
        if saveto is None:
            filename = "band.dat"
        else:
            filename = saveto
        with open(filename, "w") as f:
            for iorb in range(self.Atoms.nspinorbitals):
                for ik in range(len(kps[0])):
                    f.write("%f  %f " % (kps[1][ik], eks[ik][iorb]))
                    if with_weights:
                        for ilay in range(self.Atoms.nspinorbitals):
                            weight = Uks[ik, ilay, iorb] * \
                                Uks[ik, ilay, iorb].conj()
                            f.write("%f  " % (weight.real))
                    f.write("\n")
                f.write("\n")

    def get_dos(self,  kps_size=None, saveto=None, dos_mesh=None, eta=1e-2):
        """
        Get the density of states. Note: symmetry is not used.

        Parameters
        ----------
        dos_mesh: ndarray
          the mesh of dos. default: numpy.linspace(Emax,Emin,500),
          (Emax, Emin)= (max,min) among all the eigenvalues
        kps_size: tuple
          the size of Monkhorst_pack mesh. default: (8,8,8)
        saveto: string
          name of the file to save data.
        eta: float
          broadening factor. current only Gaussian braodening available,
          default: 1e-2

        Returns
        -------
        dos: density of states

        Examples::

        >>># dos of 2D square lattice
        >>>aTB=TB.gallery()
        >>>aTB.get_dos(kps_size=(400,400,1))
        """

        if kps_size is None:
            kps_size = (8, 8, 8)
        kps = kpoints.monkhorst_pack(kps_size)
        eks = [self.eigens(i, kpt)[0] for i, kpt in enumerate(kps)]

        if dos_mesh is None:
            dos_mesh = numpy.linspace(eks.min(), eks.max(), 500)
        if saveto is None:
            filename = "dos.dat"
        else:
            filename = saveto
        dos = numpy.zeros(len(dos_mesh), dtype=numpy.float)
        for ek in numpy.nditer(eks):
            dos += numpy.exp(-(ek - dos_mesh)**2 / 2.0 / eta**2)
        dos *= 1.0 / numpy.sqrt(2 * numpy.pi) / eta
        dos /= len(kps)

        with open(filename, "w") as f:
            for idx in xrange(len(dos_mesh)):
                f.write("%f  %f \n" % (dos_mesh[idx], dos[idx]))

    def add_spindegeneracy(self):
        """
        Add spin degeneracy to a non-spin TB hamiltonian.
        Note simply increase the number of orbitals and increase the size
        of Hr to 2x2 block diagonal form. up and down spin
        in block-diagonal form.

        Returns
        -------
        TB: AtomsTB
          A new AtomsTB object with spin degeneracy.
        """
        atoms = self.Atoms.copy()
        assert not self.Atoms.spindeg, \
                " Error: spin degeneray is already considered!"
        assert not self.Atoms.spinorbit, \
                " Error: Cann't add spin degeneracy to spin-orbit orbitals!"
        atoms.set_orbitals_spindeg(self.Atoms.orbitals, spindeg=True)
        norb = atoms.nspinorbitals
        if self.Hr is not None:
            Hr = {}
            for iR in self.Hr:
                Hr[iR] = numpy.zeros((norb, norb), \
                        dtype=type(self.Hr[iR][0, 0]))
                Hr[iR][0:norb / 2, 0:norb / 2] = self.Hr[iR][:, :]
                Hr[iR][norb / 2:, norb / 2:] = self.Hr[iR][:, :]
        else:
            Hr = None
        return TB(atoms, Hr)


    def save_bareham(self, kpts, ngroup=1):
        nk_per_group = len(kpts)/ngroup + 1
        ik_start_list = [i*nk_per_group for i in range(ngroup)]
        for igroup, ik_start in enumerate(ik_start_list):
            with h5py.File('BAREHAM_'+str(igroup)+'.h5', 'w') as f:
                ik_end = min(ik_start+nk_per_group, len(kpts))
                for ik in range(ik_start, ik_end):
                    ek, Uk = self.eigens(ik, kpts[ik])
                    # Upper case: Fortran convention
                    f['/IKP_'+str(ik+1)+'/ek0'] = ek
                    f['/IKP_'+str(ik+1)+'/ISYM_1/HK0'] = self.hk.T
                    f['/IKP_'+str(ik+1)+'/T_PSIK0_TO_HK0_BASIS'] = Uk.T



if __name__ == "__main__":
    # The following is a simple test for the above codes.

    # AtomsTB object
    # set up an AtomsTB object. one band without spin degeneracy.
    a = AtomsTB("N", [(0, 0, 0)], cell=(1, 1, 1))
    a.set_orbitals_spindeg()

    # sqare lattice
    # set up a TB object and the hoppings. This corresponds to a 2D squared
    # lattice with nearest-neighbor hopping. This is the default one in
    # TB.gallery()
    aTB = TB(a)
    aTB.set_hop([((0, 1, 0), 0, 0, -1),
                 ((1, 0, 0), 0, 0, -1),
                 ((0, -1, 0), 0, 0, -1),
                 ((-1, 0, 0), 0, 0, -1)])

    # bands and dos
    # set special k points for bands
    kG = [0, 0, 0]
    kX = [0.5, 0, 0]
    kM = [0.5, 0.5, 0]
    # set up a ase.dft.kpoints kpath object
    kps = kpoints.get_bandpath([kG, kX, kM, kG], a.cell)
    # get band structure of a square lattice
    aTB.get_bandstructure(kps, saveto="pcell_band.dat")



from __future__ import print_function

import h5py, numpy, warnings, sys
from scipy.linalg import block_diag
from mpi4py import MPI
from pymatgen.core import Structure
from pymatgen.symmetry.bandstructure import HighSymmKpath
from pymatgen.electronic_structure.bandstructure import BandStructureSymmLine
from pymatgen.electronic_structure.plotter import BSPlotter
from pymatgen.electronic_structure.core import Spin
from builtins import zip,range
import itertools as it
from pygrisb.iface.wanniertb import w90


def get_csh2sab():
    '''get transformation from complex spherical harmonics basis to
    g-risb symmetry adapted basis.
    '''
    csh2sab_list = []
    with h5py.File("GPARAM.h5", "r") as f:
        imap_list = f["/IMAP_IMP"][()]
        for i,imap in enumerate(imap_list):
            if i == imap-1:
                csh2sab_list.append(\
                        f["/IMPURITY_{}/DB_TO_SAB".format(i+1)][()].T)
            else:
                csh2sab_list.append(csh2sab_list[imap-1])
    return csh2sab_list


def get_wan2sab():
    '''get transformation from wannier basis to cygutz symmetry adapted basis.
    '''
    csh2sab_list = get_csh2sab()
    with h5py.File("ginit.h5", "r") as f:
        wan2csh = f["/u_wan2csh"][()]
        spin_orb = f["/usrqa/spin_orbit_coup"][()]
    if 'n' in spin_orb.lower():
        iso = 1
    else:
        iso = 2
    csh2sab = []
    for u1 in csh2sab_list:
        n1 = u1.shape[0]/(3-iso)
        csh2sab.append(u1[:n1, ::3-iso])
    csh2sab = block_diag(*csh2sab)
    wan2sab = wan2csh.copy()
    n1 = csh2sab.shape[0]
    wan2sab[:,:n1] = wan2csh[:,:n1].dot(csh2sab)
    return wan2sab


def get_gloc_in_wannier_basis():
    '''get the gutzwiller local matrices, including r, lambda, nr, and nphy
    from grisb calculation.
    '''
    # read from cygutz output
    with h5py.File("GLOG.h5", "r") as f:
        # renormalized local one-body part of the quasiparticle hamiltonian
        lambda_list = f["/BND_LAMBDA"][()].swapaxes(1,2)
        # quasiparticle renormalization matrix
        r_list = f["/BND_R"][()].swapaxes(1,2)
        # local density part to be subtracted in density calculations
        nr_list = f["/BND_NRL"][()].swapaxes(1,2)
        # physical density matrix to be added in density calculations
        nphy_list = f["/BND_NPHY"][()].swapaxes(1,2)

    # get transformation from wannier basis to cygutz symmetry adapted basis.
    wan2sab = get_wan2sab()
    # convert to wannier basis in each spin block.
    lam2 = []
    r2 = []
    nr2 = []
    nphy2 = []
    n2 = wan2sab.shape[0]
    for rmat,lam,nr,nphy in zip(r_list, lambda_list, nr_list, nphy_list):
        n1 = rmat.shape[0]
        if n2 > n1:
            rmat = block_diag(rmat, numpy.eye(n2-n1, dtype=numpy.complex))
            zeromat = numpy.zeros((n2-n1, n2-n1), numpy.complex)
            lam = block_diag(lam, zeromat)
            nr = block_diag(nr, zeromat)
            nphy = block_diag(nphy, zeromat)
        r2.append(wan2sab.dot(rmat).dot(wan2sab.T.conj()))
        lam2.append(wan2sab.dot(lam).dot(wan2sab.T.conj()))
        nr2.append(wan2sab.conj().dot(nr).dot(wan2sab.T))
        nphy2.append(wan2sab.conj().dot(nphy).dot(wan2sab.T))
    return numpy.asarray(r2), numpy.asarray(lam2), \
            numpy.asarray(nr2), numpy.asarray(nphy2)


def get_h1e_in_wannier_basis():
    '''get the h1e-matrix.
    '''
    with h5py.File("GPARAM.h5", "r") as f:
        num_imp = f["/num_imp"][0]
        ispin = f["/ispin"][0]
    # read wannier to csh-basis transformation.
    with h5py.File("ginit.h5", "r") as f:
        wan2csh = f["/u_wan2csh"][()]
    n2 = wan2csh.shape[0]
    h1e_list = []
    with h5py.File("GPARAMBANDS.h5", "r") as f:
        for isp in range(ispin):
            h1e = []
            for i in range(num_imp):
                h1e.append(f["/IMPURITY_{}/H1E_SPIN{}".format(\
                        i+1, isp+1)][()].T)
            h1e = block_diag(*h1e)
            n1 = h1e.shape[0]
            if n2 > n1:
                h1e = block_diag(h1e, numpy.zeros((n2-n1,n2-n1), \
                        dtype=numpy.complex))
            h1e_list.append(wan2csh.dot(h1e).dot(wan2csh.T.conj()))
    return h1e_list


def get_structure():
    with h5py.File("ginit.h5", "r") as f:
        lattice = f["/struct/cell"][()]
        species = f["/struct/symbols"][()]
        coords = f["/struct/scaled_positions"][()]
    return Structure(lattice=lattice, species=species, coords=coords)


def get_bands(kpoints, gmodel=None, wfwannier_list=None,
        bnd_es_in=None, mode="tb"):
    if mode == "risb":
        r_mat, lam_mat, _, _ = get_gloc_in_wannier_basis()
        h1_mat = get_h1e_in_wannier_basis()
        ispin = r_mat.shape[0]
        with h5py.File("GLOG.h5", "r") as f:
            efermi = f["/e_fermi"][0]
    else:
        ispin = 1
        efermi = 0.
    bnd_es = []
    bnd_vs = []
    for isp in range(ispin):
        if mode == "risb":
            ispp = min(isp, len(h1_mat))
        bnd_es.append([])
        bnd_vs.append([])
        for ik, kpt in enumerate(kpoints):
            if gmodel is not None:
                hmat = gmodel._gen_ham(kpt,isp)
            else:
                hmat = wfwannier_list[isp][ik].T.conj().dot(\
                        numpy.diag(bnd_es_in[isp][ik])).dot(\
                        wfwannier_list[isp][ik])
            if mode == "risb":
                hmat -= h1_mat[ispp]
                hmat = r_mat[isp].dot(hmat).dot(r_mat[isp].T.conj())
                hmat += lam_mat[isp]
            evals, evecs = numpy.linalg.eigh(hmat)
            evals -= efermi
            bnd_es[isp].append(evals)
            bnd_vs[isp].append(evecs)
    return numpy.asarray(bnd_es), numpy.asarray(bnd_vs)


def get_symkpath(atol=1.e-6):
    struct = get_structure()
    kpath = HighSymmKpath(struct)
    # check warning and perform transformation if needed.
    if not numpy.allclose(kpath._structure.lattice.matrix,
            kpath._prim.lattice.matrix, atol=atol):
        warnings.warn("Input structure does not match expected standard "
                "primitive! Try k-path transformation.")
        ktrans = kpath._prim.lattice.reciprocal_lattice.matrix.dot(\
                numpy.linalg.inv(kpath._structure.lattice.\
                reciprocal_lattice.matrix))
        for kname in kpath.kpath["kpoints"]:
            kpath.kpath["kpoints"][kname] = \
                    kpath.kpath["kpoints"][kname].dot(ktrans)
    return kpath


def get_gmodel(wpath="../wannier", wprefix="wannier"):
    wannier90 = w90(wpath, wprefix)
    gmodel = wannier90.model()
    return gmodel


def mpiget_bndev(k_list, gmodel=None, wfwannier_list=None, \
        bnd_es_in=None, mode="tb"):
    comm = MPI.COMM_WORLD
    rank = comm.Get_rank()
    ncpu = comm.Get_size()
    nktot = len(k_list)
    nk_cpu = nktot//ncpu
    if nk_cpu*ncpu < nktot:
        nk_cpu += 1
    iksta = rank*nk_cpu
    ikend = min((rank+1)*nk_cpu, nktot)
    kvec_loc = k_list[iksta: ikend]
    if wfwannier_list is not None:
        wfwannier_list = wfwannier_list[:, iksta:ikend, :, :]
        bnd_es_in = bnd_es_in[:, iksta:ikend, :]
    bnd_es, bnd_vs = get_bands(kvec_loc, gmodel=gmodel, \
            wfwannier_list=wfwannier_list, bnd_es_in=bnd_es_in, mode=mode)
    if rank != 0:
        comm.Send(bnd_es, dest=0, tag=rank)
    else:
        for icpu in range(1, ncpu):
            nk_loc = min(nk_cpu, nktot-icpu*nk_cpu)
            _bnd_es = numpy.zeros((bnd_es.shape[0], nk_loc, bnd_es.shape[2]), \
                    dtype=numpy.float)
            comm.Recv(_bnd_es, source=icpu, tag=icpu)
            bnd_es = numpy.concatenate((bnd_es, _bnd_es), axis=1)
        assert bnd_es.shape[1] == nktot, "error in merging bnd_es!"
    return bnd_es, bnd_vs


def get_wannier_den_matrix_risb(bnd_vs, ferwes, wk, nktot):
    comm = MPI.COMM_WORLD
    rank = comm.Get_rank()
    ncpu = comm.Get_size()
    nk_cpu = nktot//ncpu
    if nk_cpu*ncpu < nktot:
        nk_cpu += 1
    r_mat, _, nr_mat, nphy_mat = get_gloc_in_wannier_basis()
    with h5py.File("GPARAMBANDS.h5", "r") as f:
        ispin_dft = f["/ispin"][0]
        iso = f["/iso"][0]
    ispin_risb = r_mat.shape[0]
    # spin factor
    f_ispin = 3-max(iso, ispin_risb)
    wan_den = []
    # total number of electrons to be compared.
    sum_elec1 = numpy.sum(ferwes)
    sum_elec2 = 0.
    for isp in range(ispin_risb):
        if isp <= ispin_dft:
            wan_den.append([])
        else:
            wan_den = numpy.asarray(wan_den)
        for ik, bndvk1, ferwek1, wk1 in zip(it.count(),
                bnd_vs[isp], ferwes[isp], wk):
            # notice the convention a bit different from cygutz.
            # <a|psi>f<psi|b>
            afb = bndvk1.dot(numpy.diag(ferwek1/wk1/f_ispin)).dot(\
                    bndvk1.T.conj())
            # R^\dagger_{A,a} * <a|psi>f<psi|b> * R_{b,B}
            rdafbr = r_mat[isp].T.conj().dot(afb).dot(r_mat[isp])
            dmk = rdafbr
            # \rho_{A,B} = R^\dagger_{A,a} * <a|psi>f<psi|b> * R_{b,B}
            #            +(n_phys.^{A,B} - n_{sub.}^{A,B})
            dmk += (nphy_mat[isp]-nr_mat[isp]).T
            sum_elec2 += dmk.trace()*wk1*f_ispin
            if isp <= ispin_dft:
                wan_den[-1].append(dmk)
            else:
                wan_den[-1][ik] += dmk
                wan_den[-1][ik] *= 0.5
    sum_elec_all1 = comm.reduce(sum_elec1)
    sum_elec_all2 = comm.reduce(sum_elec2)
    if rank == 0:
        elec_diff = sum_elec_all2-sum_elec_all1
        if numpy.abs(elec_diff) > 1.e-4:
            warnings.warn("sum_ferwt = {} vs sum_kswt = {}!". \
                    format(sum_elec1, sum_elec2))
    wan_den = numpy.asarray(wan_den)
    # merge wan_den to master node
    if rank != 0:
        comm.Send(wan_den, dest=0, tag=rank)
    else:
        for icpu in range(1, ncpu):
            nk_loc = min(nk_cpu, nktot-icpu*nk_cpu)
            _wan_den = numpy.zeros((wan_den.shape[0], nk_loc, \
                    wan_den.shape[2], wan_den.shape[3]), dtype=numpy.complex)
            comm.Recv(_wan_den, source=icpu, tag=icpu)
            wan_den = numpy.concatenate((wan_den, _wan_den), axis=1)
        assert wan_den.shape[1] == nktot, "error in merging wan_den!"
    return wan_den


def get_bands_symkpath(efermi=0., mode="tb"):
    comm = MPI.COMM_WORLD
    rank = comm.Get_rank()
    if rank == 0:
        gmodel = get_gmodel()
        kpath = get_symkpath()
        nk = (len(kpath.kpath["kpoints"])-1)*12
        k_vec, k_dist, k_node = gmodel.k_path(kpath.kpath["kpoints"]. \
                values(), nk)
    else:
        gmodel = kpath = k_vec = k_dist = k_node = None
    gmodel = comm.bcast(gmodel, root=0)
    kpath = comm.bcast(kpath, root=0)
    k_vec = comm.bcast(k_vec, root=0)
    k_dist = comm.bcast(k_dist, root=0)
    k_node = comm.bcast(k_node, root=0)
    bnd_es, bnd_vs = mpiget_bndev(k_vec, gmodel=gmodel, mode=mode)
    # prepare the args for pymatgen bs class.
    if rank == 0:
        eigenvals = {}
        eigenvals[Spin.up] = bnd_es[0].T
        if len(bnd_es) == 2:
            eigenvals[Spin.down] = bnd_es[1].T
        bs = BandStructureSymmLine(k_vec, eigenvals, \
                kpath._structure.lattice.reciprocal_lattice, \
                efermi, kpath.kpath["kpoints"])
    else:
        bs = None
    return bs


def plot_bandstructure():
    if "-h" in sys.argv:
        print("usage: complot_bands.py [-g] [-f fname] [-el emin] [-eh emax]")
        sys.exit()

    if "-g" in sys.argv:
        mode = "risb"
    else:
        mode = "tb"
    bs = get_bands_symkpath(mode=mode)
    comm = MPI.COMM_WORLD
    rank = comm.Get_rank()
    if rank == 0:
        bsplot = BSPlotter(bs)
        if "-f" in sys.argv:
            fname = sys.argv[sys.argv.index("-f")+1]
            if ".pdf" not in fname:
                fname += ".pdf"
        else:
            fname = "bndstr.pdf"
        if "-el" in sys.argv:
            emin = float(sys.argv[sys.argv.index("-el")+1])
        else:
            emin = numpy.min(bs.bands.values())
        if "-eh" in sys.argv:
            emax = float(sys.argv[sys.argv.index("-eh")+1])
        else:
            emax = numpy.max(bs.bands.values())
        bsplot.save_plot(fname, img_format="pdf", ylim=(emin, emax), \
                zero_to_efermi=False)
        # better align by yourself, setting zero_to_efermi=False.



if __name__ == "__main__":
    plot_bandstructure()



from scipy.sparse import csr_matrix,coo_matrix


def h5auto_read(f, path, default=None):
    if path in f:
        return f[path][()]
    else:
        return default


def h5auto_write(f, path, data):
    if path in f:
        del f[path]
    f[path] = data


def get_csr_matrix(f, path):
    '''
    Read the csr_matrix located at path in the hdf5 file f.
    '''
    nrow = f[path + "/nrow"][0]
    ncol = f[path + "/ncol"][0]
    data = f[path + "/data"][()]
    base = f[path + "/base"][0]
    # possible one-based to zero-based
    indices = f[path + "/indices"][()] - base
    indptr = f[path + "/indptr"][()] - base
    return csr_matrix((data, indices, indptr), shape=(nrow, ncol))


def get_coo_matrix(f, path):
    '''
    Read the coo_matrix located at path in the hdf5 file f.
    '''
    nrow = f[path + "/nrow"][0]
    ncol = f[path + "/ncol"][0]
    data = f[path + "/data"][()]
    base = f[path + "/base"][0]
    # possible one-based to zero-based
    indi = f[path + "/i"][()] - base
    indj = f[path + "/j"][()] - base
    return coo_matrix((data, (indi, indj)), shape=(nrow, ncol))


def write_csr_matrix(f, path, a):
    '''
    Read the csr_matrix located at path in the hdf5 file f.
    '''
    if path in f:
        del path
    f[path + "/nrow"] = [a.shape[0]]
    f[path + "/ncol"] = [a.shape[1]]
    f[path + "/data"] = a.data
    f[path + "/base"] = [0]
    f[path + "/indices"] = a.indices
    f[path + "/indptr"] = a.indptr


def write_coo_matrix(f, path, a):
    '''
    Write the coo_matrix located at path in the hdf5 file f.
    '''
    f[path + "/nrow"] = [a.shape[0]]
    f[path + "/ncol"] = [a.shape[1]]
    f[path + "/nnz"] = [a.nnz]
    f[path + "/data"] = a.data
    f[path + "/base"] = [0]
    f[path + "/i"] = a.row
    f[path + "/j"] = a.col


def get_hs_rotations(f, imp, valences):
    '''
    Get rotation representations in Hilbert space.
    '''
    Rpr_list = []
    for val in valences:
        Rpr_list.append([])
        dim_rot = f["Impurity_{}/val_block={}/dim_rotations".format( \
                imp, val)][()]
        for i in range(dim_rot):
            Rpr_list[-1].append(get_csr_matrix(f, \
                    "/Impurity_{}/val_block={}/rotation_{}".format( \
                    imp, val, i)))
    return Rpr_list


if __name__ == "__main__":
    '''
    Test.
    '''



from __future__ import print_function
#Gutzwiller embedding Hamiltonian solver using machine learning.

import h5py, time, pickle, sys, numpy
import pygrisb.gsolver as gsolver


def get_input_soc_only(imp=1, l=3):
    '''get the embedding Hamiltonian parameters of impurity with index imp,
    given the orbital angular momentum l
    and assuming local spin-orbit interaction dominates.
    '''
    j2 = 2*l
    with h5py.File('EMBED_HAMIL_{}.h5'.format(imp), 'r') as f:
        d1 = f['/D'][0, 0]
        d2 = f['/D'][j2, j2]
        e1 = f['/H1E'][0, 0]
        e2 = f['/H1E'][j2, j2]
        l1 = f['/LAMBDA'][0,0]
        l2 = f['/LAMBDA'][j2, j2]

    # to real
    phase1 = d1/numpy.sqrt(d1.real**2 + d1.imag**2)
    phase2 = d2/numpy.sqrt(d2.real**2 + d2.imag**2)
    # to positive
    if d1*numpy.conj(phase1).real > 0:
        phase1 *= -1.0
    if d2*numpy.conj(phase2).real > 0:
        phase2 *= -1.0

    d1 *= numpy.conj(phase1)
    d2 *= numpy.conj(phase2)

    delta = (e1+e2+l1+l2)/4.0
    delta1 = (e1-e2)/2.0
    delta2 = (l1-l2)/2.0
    eshift = l1*2*l + l2*(2*l + 2)
    x_list = numpy.array([[d1, d2, delta, delta1, delta2]])
    if numpy.max(numpy.abs(x_list.imag)) > 1.e-8:
        print(' warning: imaginary part detected in ml input!')
    x_list = x_list.real
    return x_list, eshift, [phase1, phase2]


def driver_gs_ml(l=3):
    '''driver for the machines learning based Gutzwiller solver.
    '''
    start_time = time.clock()

    imp = 1
    if '-i' in sys.argv:
        imp = int(sys.argv[sys.argv.index('-i')+1])
    if '-l' in sys.argv:
        l = int(sys.argv[sys.argv.index('-l')+1])
    if l == 3:
        subd = 'f_so_v2'
    else:
        raise ValueError(' l = {} not available!'.format(l))

    print(' solving emb. hamil. for impurity {}\n'.format(imp) +\
            ' with l = {} based on machine learning.'.format(l))

    dpath = gsolver.__path__[0]
    inp, eshft, phase = get_input_soc_only(imp=imp, l=l)
    with open('{}/{}/kr_fso.pkl'.format(dpath, subd), \
                'rb') as f:
        kr = pickle.load(f)
    res = kr.predict(inp)[0, :]

    # density matrix
    na2 = (2*l+1)*2
    na4 = na2*2
    dm = numpy.zeros([na4, na4], dtype=numpy.complex)
    for i in range(2*l):
        dm[i, i] = res[0]
        dm[i + na2, i + na2] = res[2]
        dm[i, i + na2] = res[4]*numpy.conj(phase[0])
        dm[i + na2, i] = numpy.conj(res[4])*phase[0]
    for i in range(2*l, na2):
        dm[i, i] = res[1]
        dm[i + na2, i + na2] = res[3]
        dm[i, i + na2] = res[5]*numpy.conj(phase[1])
        dm[i + na2, i] = numpy.conj(res[5])*phase[1]
    etot = kr.predict(inp)[0, 6]
    # convert to molecule energy
    etot -= eshft

    with h5py.File('EMBED_HAMIL_RES_{}.h5'.format(imp), 'w') as f:
        f['/DM'] = dm.T
        f['/emol'] = [etot.real]

    end_time = time.clock()

    print(' total time: {}'.format(end_time - start_time))
    sys.exit(0)



if __name__ == '__main__':
    driver_gs_ml()




from __future__ import print_function

import unittest
import pickle

with open('mol.pickle', 'rb') as f:
    mol = pickle.load(f)

from pymatgen.symmetry.analyzer import PointGroupAnalyzer
analyzer = PointGroupAnalyzer(mol)
print(" sch_symbol = ", analyzer.sch_symbol)

from pymatgen.symmetry.analyzer import generate_full_symmops
symmops = generate_full_symmops(analyzer.symmops, tol=1.e-7)
print(" num symmops = ", len(symmops))


class KnowValues(unittest.TestCase):
    def test_PointGroupAnalyzer(self):
        self.assertTrue(analyzer.sch_symbol == 'D4h')
        self.assertTrue(len(symmops) == 16)


if __name__ == "__main__":
    print("Tests for pymatgen.")
    unittest.main()




from __future__ import print_function


def print_2d_array(dm, msg):
    print(msg)
    print('real part')
    for row in dm:
        print(''.join('%9.4f' % (x.real) for x in row))
    print('imag part')
    for row in dm:
        print(''.join('%9.4f' % (x.imag) for x in row))



# Always prefer setuptools over distutils
from setuptools import setup, find_packages
# To use a consistent encoding
from codecs import open
from os import path

# here = path.abspath(path.dirname(__file__))
# Get the long description from the README file
# with open(path.join(here, 'README.rst'), encoding='utf-8') as f:
#    long_description = f.read()

setup(
    name='pygrisb',
    version='2019.2.24.1',
    scripts=['readme.py'],
    author="Yongxin Yao, Nicola Lanata, Xiaoyu Deng",
    author_email="cygutz@gmail.com",
    description='Python libraries for CyGutz package',
    url="https://gitlab.com/yxphysics/pygrisb",
    license='GPL',
    classifiers=[
        # How mature is this project? Common values are
        #   3 - Alpha
        #   4 - Beta
        #   5 - Production/Stable
        'Development Status :: 3 - Alpha',

        # Indicate who your project is intended for
        'Intended Audience :: Developers',
        'Topic :: Software Development :: Build Tools',

        # Pick your license as you wish (should match "license" above)
        'License :: OSI Approved :: GNU Lesser General Public License v3 or later (LGPLv3+)',

        # Specify the Python versions you support here.
        'Programming Language :: Python :: 3.7',
    ],
    packages=["pygrisb"],
    package_dir={"pygrisb": "pygrisb"},
    #package_dir={"pygrisb": ["pygrisb/*","pygrisb/*/*","pygrisb/*/*/*",
    #        "pygrisb/*/*/*/*","pygrisb/*/*/*/*/*","pygrisb/*/*/*/*/*/*"]
    #        },
    install_requires=['numpy', 'scipy', "pymatgen", "mpi4py", "future", \
            "matplotlib", 'h5py', 'ase'],
)



from __future__ import print_function

import h5py
from pygrisb.gutz.init import initialize


def batch_initialize(cell, scaled_positions, symbols, case=None,
        crystal_field='y', dist_cut=-1, full_orbital_polarization='y',
        idx_equivalent_atoms=None, iembeddiag=-1, ldc=1, lnewton=0,
        spin_orbit_coup='n', spin_polarization='n', u_matrix_type=1,
        unique_corr_symbol_list=None, unique_df_list=None,
        unique_j_list_ev=None, unique_nf_list=None,
        updn_full_list=None,
        unique_u_list_ev=None, unit='eV'):
    '''initialize *CyGutz* calculation by directly providing values
    of the list of arguments.

    Parameters:

    * cell: 3x3 matrix
        Unit cell vectors.
    * scaled_positions: ist of scaled-positions
        Atomic positions given in units of the unit cell.
    * symbols: list of str
        a list of symbols. E.g., ['H', 'H', 'O']
    * case: str
        case name for Wien2k.
    * crystal_field: str
        'y' for considering crystal field, and 'n' for not
    * dist_cut: real number
        cut-off distance for extracting finite fragment centered at an atom
        from the infinite lattice. If default (-1), the code would determine
        it automatically.
    * full_orbital_polarization: str
        y' for breaking all the possible orbital symmetry, and 'n' for not.
    * idx_equivalent_atoms: list of integers
        Indices for equivalent atoms. E.g., 0 0 0 1 1' means 1-3 and 4-5
        are two inequivalent atoms.
    * iembeddiag: integer
        flag for the method to solve embedding Hamiltonian.

        * -3: Valence truncation ED for S=0 (spin-singlet)
        * -1: Valence truncation ED
        * 10: Hartree-Fock (LDA+U)

    * ldc: integer
        flag for Coulomb interaction double counting.

        * 12: Recommended for LDA+G-RISB calculations.
            FLL double counting.
            (updating Vdc at each charge iteration,
            initial n0 to be provided.)
        *  1: FLL double counting potential.
            (n_0 self-consistently determined.)
        *  0: No double counting (useful for models).

    * lnewton: integer
        flag for the method to solve G-RISB equations.

        *  0: Modified Powell hybrid method (HYDRD1).
        * -1: Broyden method.

    * spin_orbit_coup: str
        'y' for considering spin-orbit coupling, and 'n' for not.
    * spin_polarization: str
        'y' for spin-polarization, and 'n' for not
    * u_matrix_type: integer
         flag for the parametrize Coulomb U-matrix.

         * 1: Slater-Condo parametrization.
         * 2: Kanamori parametrization (useful for models).
         * 0: Manual input.

    * unique_corr_symbol_list: list of str
        list of unique correlated atom symbols
    * unique_df_list: list of str
        list of s/p/d/f orbitals
        for the list of unique correlated atom symbols
    * unique_j_list_ev: list of real
        list of Hund's parameter J
        for the list of unique correlated atom symbols
    * unique_u_list_ev: list of real
        list of Hubbard U
        for the list of unique correlated atom symbols
    * unique_nf_list: list of real
        list of initial impurity orbital occupations
        for the list of unique correlated atom symbols
    * unit: str
        label for the unit system: 'eV' or 'rydberg'.

    Results:

    finish the initializztion for *CyGutz* accordingly and create
    the ``GPARAM.h5`` file.
    '''

    with h5py.File('ginit.h5', 'w') as f:
        if case is not None:
            f['/struct/case'] = case
        f['/struct/cell'] = cell
        f['/struct/scaled_positions'] = scaled_positions
        f['/struct/symbols'] = symbols
        f['/usrqa/crystal_field'] = crystal_field
        f['/usrqa/dist_cut'] = dist_cut
        f['/usrqa/full_orbital_polarization'] = full_orbital_polarization
        if idx_equivalent_atoms is None:
            idx_equivalent_atoms = range(len(symbols))
        f['/usrqa/idx_equivalent_atoms'] = idx_equivalent_atoms
        f['/usrqa/iembeddiag'] = iembeddiag
        f['/usrqa/ldc'] = ldc
        f['/usrqa/lnewton'] = lnewton
        f['/usrqa/spin_orbit_coup'] = spin_orbit_coup
        f['/usrqa/spin_polarization'] = spin_polarization
        if 'y' in spin_polarization:
            if updn_full_list is None:
                updn_full_list = [1 for s in symbols]
            f['/usrqa/updn_full_list'] = updn_full_list
        f['/usrqa/u_matrix_type'] = u_matrix_type
        if unique_corr_symbol_list is None:
            unique_corr_symbol_list = list(set(symbols))
        f['/usrqa/unique_corr_symbol_list'] = unique_corr_symbol_list
        if unique_df_list is None:
            unique_df_list = ['s' for x in unique_corr_symbol_list]
        f['/usrqa/unique_df_list'] = unique_df_list
        if unique_j_list_ev is None:
            unique_j_list_ev = [0. for x in unique_corr_symbol_list]
        f['/usrqa/unique_j_list_ev'] = unique_j_list_ev
        if unique_u_list_ev is None:
            unique_u_list_ev = [0. for x in unique_corr_symbol_list]
        f['/usrqa/unique_u_list_ev'] = unique_u_list_ev
        if unique_nf_list is None:
            unique_nf_list = [1. for x in unique_corr_symbol_list]
        f['/usrqa/unique_nf_list'] = unique_nf_list
        f['/usrqa/unit'] = unit

    initialize()



from __future__ import print_function
import matplotlib.pyplot as plt
try:
    from builtins import zip
except:
    pass

import numpy, h5py
from pygrisb.basic.splot import colors


def s_to_index(s):
    '''get integer index for s, which can either be integer or halves.
    '''
    if 0.49 < s % 1 < 0.51: # half s, 1/2, 3/2, ...
        return int(s - 0.49)
    elif 0 <= s % 1 < 0.01 or 0.99 < s % 1: # integer s, 0, 1, ...
        return int(s + 0.1)
    else:
        raise ValueError(' s = {}'.format(s))


def get_group_weights(weights_list, n_labels_list, s_labels_list):
    '''group weights according to valence n and s quantum number.
    '''
    nmin = numpy.min(n_labels_list[0])
    nmax = numpy.max(n_labels_list[0])
    smax = numpy.max(s_labels_list[0])
    ns = s_to_index(smax)

    g_weights_list = []
    for weights, n_labels, s_labels in zip(weights_list, n_labels_list,
            s_labels_list):
        g_weights_list.append(numpy.zeros([nmax-nmin+1, ns+1]))
        for wt, n, s in zip(weights, n_labels, s_labels):
            if wt < 1.e-7:
                continue
            try:
                i = s_to_index(s)
            except ValueError:
                print(' wt = {}: to be ignored!'.format(wt))
                continue
            g_weights_list[-1][n-nmin, i] += wt
    return g_weights_list


def hist_ns(weights_list, n_labels_list, s_labels_list,
        amlabel='S', tol=1.e-2):
    '''generate histogram plot for the multiple local reduced density matrix
    with label n and s(j).
    '''
    x_ticks = []
    label_ticks = []
    icolor = 0
    color_map = [{}, {}]
    nmin = numpy.min(n_labels_list[0])
    g_weights_list = get_group_weights(weights_list, n_labels_list,
            s_labels_list)
    idx = -1
    fig, ax = plt.subplots(figsize=(6,3))
    g_weights_list = numpy.asarray(g_weights_list)
    rwidth = 1./(1+g_weights_list.shape[0])

    cmap = plt.get_cmap('plasma')

    nalpha_start = -1
    for n in range(g_weights_list.shape[1]):
        wmax = numpy.max(g_weights_list[:, n, :])
        if wmax >= tol:
            if nalpha_start == -1:
                nalpha_start = n
            else:
                nalpha_end = n

    for n in range(g_weights_list.shape[1]):
        idx_start = idx
        for s in range(g_weights_list.shape[2]):
            wmax = numpy.max(g_weights_list[:, n, s])
            jn = (n+nmin) % 2
            if wmax >= tol:
                idx += 1
                if s in color_map[jn]:
                    color = color_map[jn][s]
                    label = None
                else:
                    icolor += 1
                    color = color_map[jn][s] = colors[icolor]
                    if jn == 0:
                        label = '{}={}'.format(amlabel, s)
                    else:
                        label = '{}={}/2'.format(amlabel, 2*s+1)
                for i, w in enumerate(g_weights_list[:, n, s]):
                    ax.hist([idx-0.5+rwidth*(i+1)], 1, weights=[w],
                            rwidth=rwidth*0.9, align='mid',
                            color=color, label=label)
                    label = None

        if idx > idx_start:
            ax.axvspan(idx_start+0.5, idx+0.5,
                    facecolor=cmap(float(n-nalpha_start) \
                    /(nalpha_end-nalpha_start)),
                    zorder=-1, alpha=0.2, ls='--', ec='blue')
            x_ticks.append((idx_start+idx+1)/2.)
            label_ticks.append('N='+str(n+nmin))
    ax.set_xlim(-0.5, idx+0.5)
    ax.set_xticks(x_ticks)
    ax.set_xticklabels(label_ticks)
    ax.legend()
    ax.set_ylabel('Probability')
    fig.tight_layout()
    plt.show()
    fig.savefig('hist_s.pdf')



if __name__=='__main__':
    weights_list = []
    n_labels_list = []
    s_labels_list = []
    for i in range(2):
        with h5py.File('multiplets_{}.h5'.format(i), 'r') as f:
            weights, n_labels, s_labels = f['/impurity_1/weights'][()], \
                    f['/impurity_1/n_labels'][()], \
                    f['/impurity_1/s_labels'][()]
            weights_list.append(weights)
            n_labels_list.append(n_labels)
            s_labels_list.append(s_labels)
    hist_ns(weights_list, n_labels_list, s_labels_list)



from __future__ import print_function
import h5py, numpy, sys, glob
try:
    from builtins import range
except:
    pass

from pygrisb.estructure.dos import get_bands
import matplotlib.pyplot as plt


def get_greek_label(kname):
    '''return the possible greek label.
    '''
    if kname.upper() in ['LAMBDA', 'GAMMA', 'DELTA', 'SIGMA', 'THETA', \
            'XI', 'PI', 'UPSILON', 'PHI', 'PSI', 'OMEGA']:
        return '$\{}{}$'.format(kname[0].upper(), kname[1:].lower())
    else:
      return '${}$'.format(kname)


def get_k_info():
    '''get the k distance list, special k-point label and position for
    the band structure plot.
    '''
    with h5py.File('GPARAMBANDS.h5', 'r') as f:
        kx = f['/kptx'][()]
        ky = f['/kpty'][()]
        kz = f['/kptz'][()]
        kn = f['/kptname'][()]
        # reciprocal lattice vectors ordered as colomn vectors
        br = f['/recip_prim_vec'][()]

    ktick_pos = []
    ktick_label = []
    for i in range(len(kn)):
        if i == 0:
            klist = [0.]
        else:
            dk = numpy.array([kx[i]-kx[i-1], ky[i]-ky[i-1], kz[i]-kz[i-1]])
            dl = dk.dot(br).dot(br.T).dot(dk)
            klist.append(klist[i-1]+numpy.sqrt(dl))
        klabel = kn[i].strip()
        if klabel != '':
            ktick_pos.append(klist[i])
            ktick_label.append(get_greek_label(klabel))
    return klist, ktick_pos, ktick_label


def get_estimated_gap(nocc=None):
    '''Given the number of occupied bands and the band energy file,
    estimate the band gap.
    '''
    # valence band maximum
    evmax = -1.e10
    # conduction band minimum
    ecmin = 1.e10

    # if fermi level is not given, check the fermi level
    if nocc is None:
        with h5py.File('GLOG.h5') as f:
            e_fermi = f['/e_fermi'][0]

    for fband in glob.glob('GBANDS_*.h5'):
        with h5py.File(fband, 'r') as f:
            for ik in range(f['/IKP_START'][()], f['/IKP_END'][()]+1):
                ek = f['/ISPIN_1/IKP_{}/ek'.format(ik)][()]
                if nocc is None:
                    noccp = numpy.argwhere(ek < e_fermi)[-1] + 1
                    nocc = noccp[0]
                    print(' number of occupied bands = {}'.format(nocc))
                evmax = max(ek[nocc-1], evmax)
                ecmin = min(ek[nocc], ecmin)
    return max(ecmin - evmax, 0.)


def get_estimated_gaps(nocc=None):
    '''Given the number of occupied bands and the band energy file,
    estimate the direct/indirect band gap with associated k-index.
    '''
    # valence band maximum
    evmax = -1.e10
    kvmax = -1
    # conduction band minimum
    ecmin = 1.e10
    kcmin = -1
    # direct band gap value
    dgap = 1000.
    kdgap = -1

    # if fermi level is not given, check the fermi level
    if nocc is None:
        with h5py.File('GLOG.h5') as f:
            e_fermi = f['/e_fermi'][0]

    for fband in glob.glob('GBANDS_*.h5'):
        with h5py.File(fband, 'r') as f:
            for ik in range(f['/IKP_START'][()], f['/IKP_END'][()]+1):
                ek = f['/ISPIN_1/IKP_{}/ek'.format(ik)][()]

                if nocc is None:
                    noccp = numpy.argwhere(ek < e_fermi)[-1] + 1
                    nocc = noccp[0]
                    print(' number of occupied bands = {}'.format(nocc))

                if ek[nocc-1] > evmax:
                    evmax = ek[nocc-1]
                    # 0-based k-index
                    kvmax = ik - 1
                if ek[nocc] < ecmin:
                    ecmin = ek[nocc]
                    kcmin = ik - 1
                if ek[nocc] - ek[nocc-1] < dgap:
                    dgap = ek[nocc] - ek[nocc-1]
                    kdgap = ik - 1
    # indirect band gap size
    idgap = max(ecmin - evmax, 0.)
    return idgap, kvmax, kcmin, dgap, kdgap


def driver_get_estimated_gaps():
    '''script to print estimated direct/indirect band gaps.
    '''
    msg = r'''
    Script to print estimated direct/indirect band gaps.

    inline options:

        -n n: occupied number of bands.
    '''
    nocc = None
    if '-h' in sys.argv:
        print(msg)
        sys.exit()
    if '-n' in sys.argv:
        nocc = int(sys.argv[sys.argv.index('-n')+1])
    with h5py.File('GPARAMBANDS.h5', 'r') as f:
        kname = f['/kptname'][()]
    idgap, kvmax, kcmin, dgap, kdgap = get_estimated_gaps(nocc=nocc)
    print(' Direct gap = {} with k = {} {}'.format(dgap, kdgap, \
            kname[kdgap]))
    print(' Inirect gap = {} with kv = {} {} kc = {} {}'.format(idgap, \
            kvmax, kname[kvmax], kcmin, kname[kcmin]))


def plot_band_sturture(emin=-10., emax=10.):
    '''plot band structure with overall correlated orbital character
    in the assigned energy window [emin, emax] in the file `bands.pdf`.
    '''
    klist, ktick_pos, ktick_label = get_k_info()
    e_skn, psi_sksna = get_bands()
    # get total correlated orbital weight.
    psi_skn_f = numpy.einsum('...ijk,...ijk->...j', psi_sksna[...,:,:,:], \
            psi_sksna.conj()[...,:,:,:])/psi_sksna.shape[2].real
    # multiply a scaling factor
    psi_skn_f *= 20.

    # start plotting
    fig, ax = plt.subplots()
    for n in range(e_skn.shape[2]):
        ax.plot(klist, e_skn[0, :, n], 'k-')
        ax.scatter(klist, e_skn[0,:,n], s = psi_skn_f[0, :, n], \
                c = 'r', edgecolors = 'r')
    if len(e_skn) == 2:
        for n in range(e_skn.shape[2]):
            ax.plot(klist, e_skn[1, :, n], 'k--')
            ax.scatter(klist, e_skn[1,:,n], s = psi_skn_f[1, :, n], \
                    c = 'b', edgecolors = 'b')

    ax.axhline(y = 0, ls = ':', lw = 2)
    # High-symmetry lines and labels
    for x1 in ktick_pos[1:-1]:
        ax.axvline(x = x1, ls = '--')
    ax.set_xticks(ktick_pos)
    ax.set_xticklabels(ktick_label)
    ax.set_ylabel("E (eV)")
    ax.set_xlim(klist[0], klist[-1])
    ax.set_ylim(emin, emax)
    plt.title("Band structure with corr-orb-character")
    plt.show()
    fig.savefig('bands.pdf')


def driver_plot_bands():
    msg = r'''
    Script to plot band structure with
    overall correlated orbital character.

    inline options:

        -el emin: set energy mesh minimuim
        -eh emax: set energy mesh miaximum
    '''

    emin = -3.3
    emax = 5.8
    if '-h' in sys.argv:
        print(msg)
        sys.exit()
    else:
        if '-el' in sys.argv:
            emin = float(sys.argv[sys.argv.index('-el')+1])
        if '-eh' in sys.argv:
            emax = float(sys.argv[sys.argv.index('-eh')+1])

    plot_band_sturture(emin=emin, emax=emax)



if __name__=='__main__':
    plot_band_sturture(emin=-3.3, emax=5.8)



from __future__ import print_function
# modified based on U_matrix.py from TRIQS.

from itertools import product
import numpy as np
from scipy.misc import factorial as fact
from pygrisb.math.matrix_util import unitary_transform_coulomb_matrix


def U_matrix(mode, l, radial_integrals=None, U_int=None, J_hund=None, T=None):
    if 'slater' in mode:
        U_matrix = U_matrix_slater(l, radial_integrals, U_int, J_hund)
    elif 'kanamori' in mode:
        U_matrix = U_matrix_kanamori(2*l+1, U_int, J_hund)
    else:
        raise NameError(" unsupported mode!")
    u_avg, j_avg = get_average_uj(U_matrix)

    # add spin-components
    norb = U_matrix.shape[0]
    norb2 = norb*2
    Ufull_matrix = np.zeros((norb2,norb2,norb2,norb2), dtype=np.complex)
    if T is not None:
        # spin block
        Ufull_matrix[:norb,:norb,:norb,:norb] = U_matrix
        Ufull_matrix[norb:,norb:,norb:,norb:] = U_matrix
        Ufull_matrix[:norb,:norb,norb:,norb:] = U_matrix
        Ufull_matrix[norb:,norb:,:norb,:norb] = U_matrix
        print(" u-matrix: nnz in compl_sph_harm = {}".format(
                np.count_nonzero(np.abs(Ufull_matrix)>1.e-10)))
        Ufull_matrix = unitary_transform_coulomb_matrix(Ufull_matrix, T)
    else: # spin index fast
        Ufull_matrix[::2,::2,::2,::2] = U_matrix  # up, up
        Ufull_matrix[1::2,1::2,1::2,1::2] = U_matrix # dn, dn
        Ufull_matrix[::2,::2,1::2,1::2] = U_matrix # up, dn
        Ufull_matrix[1::2,1::2,::2,::2] = U_matrix # dn, up

    print(" u-matrix: nnz in final basis = {}".format(
            np.count_nonzero(np.abs(Ufull_matrix)>1.e-10)))
    return Ufull_matrix, u_avg, j_avg


# The interaction matrix in desired basis.
# U^{spherical}_{m1 m4 m2 m3} =
# \sum_{k=0}^{2l} F_k angular_matrix_element(l, k, m1, m2, m3, m4)
# H = \frac{1}{2} \sum_{ijkl,\sigma \sigma'} U_{ikjl}
# a_{i \sigma}^\dagger a_{j \sigma'}^\dagger a_{l \sigma'} a_{k \sigma}.
def U_matrix_slater(l, radial_integrals=None, U_int=None, J_hund=None):
    r"""
    Calculate the full four-index U matrix being given either
    radial_integrals or U_int and J_hund.
    The convetion for the U matrix is that used to construct
    the Hamiltonians, namely:

    .. math:: H = \frac{1}{2} \sum_{ijkl,\sigma \sigma'} U_{ikjl}
            a_{i \sigma}^\dagger a_{j \sigma'}^\dagger
            a_{l \sigma'} a_{k \sigma}.

    Parameters
    ----------
    l : integer
        Angular momentum of shell being treated
        (l=2 for d shell, l=3 for f shell).
    radial_integrals : list, optional
                       Slater integrals [F0,F2,F4,..].
                       Must be provided if U_int and J_hund are not given.
                       Preferentially used to compute the U_matrix
                       if provided alongside U_int and J_hund.
    U_int : scalar, optional
            Value of the screened Hubbard interaction.
            Must be provided if radial_integrals are not given.
    J_hund : scalar, optional
             Value of the Hund's coupling.
             Must be provided if radial_integrals are not given.

    Returns
    -------
    U_matrix : float numpy array
               The four-index interaction matrix in the chosen basis.
    """

    # Check all necessary information is present and consistent
    if radial_integrals is None and (U_int is None and J_hund is None):
        raise ValueError("U_matrix: provide either the radial_integrals" +
                " or U_int and J_hund.")
    if radial_integrals is None and (U_int is not None and J_hund is not None):
        radial_integrals = U_J_to_radial_integrals(l, U_int, J_hund)
    if radial_integrals is not None and \
            (U_int is not None and J_hund is not None):
        if len(radial_integrals)-1 != l:
            raise ValueError("U_matrix: inconsistency in l" +
                    " and number of radial_integrals provided.")
        if not np.allclose(radial_integrals,
                U_J_to_radial_integrals(l, U_int, J_hund)):
            print(" Warning: U_matrix: radial_integrals provided\n"+
            " do not match U_int and J_hund.\n"+
            " Using radial_integrals to calculate U_matrix.")

    # Full interaction matrix
    # Basis of spherical harmonics Y_{-2}, Y_{-1}, Y_{0}, Y_{1}, Y_{2}
    # U^{spherical}_{m1 m4 m2 m3} = \sum_{k=0}^{2l} F_k
    # angular_matrix_element(l, k, m1, m2, m3, m4)
    U_matrix = np.zeros((2*l+1,2*l+1,2*l+1,2*l+1),dtype=np.float)

    m_range = range(-l,l+1)
    for n, F in enumerate(radial_integrals):
        k = 2*n
        for m1, m2, m3, m4 in product(m_range,m_range,m_range,m_range):
            U_matrix[m1+l,m3+l,m2+l,m4+l] += \
                    F * angular_matrix_element(l,k,m1,m2,m3,m4)
    return U_matrix


def U_matrix_kanamori(n_orb, U_int, J_hund):
    r"""
    Calculate the Kanamori U and Uprime matrices.

    Parameters
    ----------
    n_orb : integer
            Number of orbitals in basis.
    U_int : scalar
            Value of the screened Hubbard interaction.
    J_hund : scalar
             Value of the Hund's coupling.

    Returns
    -------
    U_matrix : float numpy array
               The four-index interaction matrix in the chosen basis.
    """

    U_matrix  = np.zeros((n_orb,n_orb,n_orb,n_orb),dtype=np.float)
    m_range = range(n_orb)
    for m,mp in product(m_range,m_range):
        if m == mp:
            U_matrix[m,m,mp,mp] = U_int
        else:
            U_matrix[m,m,mp,mp] = U_int - 2.0*J_hund
            U_matrix[m,mp,mp,m] = J_hund
            U_matrix[m,mp,m,mp] = J_hund
    return U_matrix


def U_J_to_radial_integrals4(l,U,J):
    label_to_l = {'s':0, 'p':1, 'd':2, 'f':3}
    if isinstance(l,basestring):
        l = label_to_l[l_label]
    f_list = U_J_to_radial_integrals(l, U, J)
    for i in range(len(f_list),4):
        f_list.append(0.)
    return f_list


# Convert U,J -> radial integrals F_k
def U_J_to_radial_integrals(l, U_int, J_hund):
    r"""
    Determine the radial integrals F_k from U_int and J_hund.

    Parameters
    ----------
    l : integer
        Angular momentum of shell being treated
        (l=2 for d shell, l=3 for f shell).
    U_int : scalar
            Value of the screened Hubbard interaction.
    J_hund : scalar
             Value of the Hund's coupling.

    Returns
    -------
    radial_integrals : list
                       Slater integrals [F0,F2,F4,..].
    """

    F = np.zeros((l+1),dtype=np.float)
    F[0] = U_int
    if l == 0:
        pass
    elif l == 1:
        F[1] = J_hund * 5.0
    elif l == 2:
        F[1] = J_hund * 14.0 / (1.0 + 0.625)
        F[2] = 0.625 * F[1]
    elif l == 3:
        F[1] = 6435.0 * J_hund / (286.0 + 195.0 * 0.668 + 250.0 * 0.494)
        F[2] = 0.668 * F[1]
        F[3] = 0.494 * F[1]
    else:
        raise ValueError(
            " U_J_to_radial_integrals: implemented only for l=0,1,2,3")
    return F


# Convert radial integrals F_k -> U,J
def radial_integrals_to_U_J(l, F):
    r"""
    Determine U_int and J_hund from the radial integrals.

    Parameters
    ----------
    l : integer
        Angular momentum of shell being treated
        (l=2 for d shell, l=3 for f shell).
    F : list
        Slater integrals [F0,F2,F4,..].

    Returns
    -------
    U_int : scalar
            Value of the screened Hubbard interaction.
    J_hund : scalar
             Value of the Hund's coupling.

    """

    U_int = F[0]
    if l == 0:
        J_Hund = 0.
    elif l == 1:
        J_hund = F[1]/5.0
    elif l == 2:
        J_hund = F[1]*(1.0 + 0.625)/14.0
    elif l == 3:
        J_hund = F[1] * (286.0 + 195.0*0.668 + 250.0*0.494)/6435.0
    else: raise ValueError("radial_integrals_to_U_J:"+
            " implemented only for l=2,3")

    return U_int,J_hund


# Angular matrix elements of particle-particle interaction
# (2l+1)^2 ((l 0) (k 0) (l 0))^2 \sum_{q=-k}^{k} (-1)^{m1+m2+q}
# ((l -m1) (k q) (l m3)) ((l -m2) (k -q) (l m4))
def angular_matrix_element(l, k, m1, m2, m3, m4):
    r"""
    Calculate the angular matrix element

    .. math::
       (2l+1)^2
       \begin{pmatrix}
            l & k & l \\
            0 & 0 & 0
       \end{pmatrix}^2
       \sum_{q=-k}^k (-1)^{m_1+m_2+q}
       \begin{pmatrix}
            l & k & l \\
         -m_1 & q & m_3
       \end{pmatrix}
       \begin{pmatrix}
            l & k  & l \\
         -m_2 & -q & m_4
       \end{pmatrix}.

    Parameters
    ----------
    l : integer
    k : integer
    m1 : integer
    m2 : integer
    m3 : integer
    m4 : integer

    Returns
    -------
    ang_mat_ele : scalar
                  Angular matrix element.

    """
    ang_mat_ele = 0
    for q in range(-k,k+1):
        ang_mat_ele += three_j_symbol((l,-m1),(k,q),(l,m3))* \
                three_j_symbol((l,-m2),(k,-q),(l,m4))* \
                (-1.0 if (m1+q+m2) % 2 else 1.0)
    ang_mat_ele *= (2*l+1)**2 * (three_j_symbol((l,0),(k,0),(l,0))**2)
    return ang_mat_ele


# Wigner 3-j symbols
# ((j1 m1) (j2 m2) (j3 m3))
def three_j_symbol(jm1, jm2, jm3):
    r"""
    Calculate the three-j symbol
    .. math::
       \begin{pmatrix}
        l_1 & l_2 & l_3\\
        m_1 & m_2 & m_3
       \end{pmatrix}.
    Parameters
    ----------
    jm1 : tuple of integers
          (j_1 m_1)
    jm2 : tuple of integers
          (j_2 m_2)
    jm3 : tuple of integers
          (j_3 m_3)
    Returns
    -------
    three_j_sym : scalar
                  Three-j symbol.
    """
    j1, m1 = jm1
    j2, m2 = jm2
    j3, m3 = jm3

    if (m1+m2+m3 != 0 or
        m1 < -j1 or m1 > j1 or
        m2 < -j2 or m2 > j2 or
        m3 < -j3 or m3 > j3 or
        j3 > j1 + j2 or
        j3 < abs(j1-j2)):
        return .0

    three_j_sym = -1.0 if (j1-j2-m3) % 2 else 1.0
    three_j_sym *= np.sqrt(fact(j1+j2-j3)*fact(j1-j2+j3)* \
            fact(-j1+j2+j3)/fact(j1+j2+j3+1))
    three_j_sym *= np.sqrt(fact(j1-m1)*fact(j1+m1)*fact(j2-m2)* \
            fact(j2+m2)*fact(j3-m3)*fact(j3+m3))

    t_min = max(j2-j3-m1,j1-j3+m2,0)
    t_max = min(j1-m1,j2+m2,j1+j2-j3)

    t_sum = 0
    for t in range(t_min,t_max+1):
        t_sum += (-1.0 if t % 2 else 1.0)/(fact(t)*fact(j3-j2+m1+t)* \
                fact(j3-j1-m2+t)*fact(j1+j2-j3-t)*fact(j1-m1-t)*fact(j2+m2-t))

    three_j_sym *= t_sum
    return three_j_sym


def get_average_uj(v2e):
    m_range = range(v2e.shape[0])
    u_avg = 0; j_avg = 0
    isum_u = 0; isum_j = 0
    for i, j in product(m_range, m_range):
        u_avg += v2e[i, i, j, j]
        isum_u += 1
        if i != j:
            j_avg += v2e[i, i, j, j] - v2e[i, j, j, i]
            isum_j += 1
    u_avg /= isum_u
    if isum_j > 0:
        j_avg = u_avg - j_avg/isum_j
    return u_avg, j_avg


def get_v2e_list(lhub, l_list, imap_list, utrans_list, u_list=None,
        j_list=None, f_list=None):
    mode_list = ['manual', 'slater-condon', 'kanamori', 'slater-condon']
    if lhub > 0:
        v2e_list = []
        u_avg_list = []
        j_avg_list = []

        for i, imap in enumerate(imap_list):
            if i > imap:
                v2e_list.append(v2e_list[imap])
                u_avg_list.append(u_avg_list[imap])
                j_avg_list.append(j_avg_list[imap])
                continue
            assert len(l_list[i]) == 1, " more than one l with lhub>0!"
            l_imp = l_list[i][0]
            utrans = utrans_list[i]

            if lhub in [1, 2]:
                v2e, u_avg, j_avg = U_matrix(mode_list[lhub], l_imp,
                        U_int=u_list[i],
                        J_hund=j_list[i], T=utrans)
            else:
                v2e, u_avg, j_avg = U_matrix(mode_list[lhub], l_imp,
                        radial_integrals=f_list[i][:l_imp+1], T=utrans)

            v2e_list.append(v2e)
            u_avg_list.append(u_avg)
            j_avg_list.append(j_avg)
    else:
        v2e_list = None
        u_avg_list = None
        j_avg_list = None

    return v2e_list, u_avg_list, j_avg_list


if __name__ == '__main__':
    l = 2
    coulomb_matrix = U_matrix('slater', l, U_int=6.7, J_hund=0.67)
    m_range = range(2*l+1)
    with open('coulomb_matrixp.txt','w') as f:
        for m1,m2,m3,m4 in product(m_range,m_range,m_range,m_range):
            if np.abs(coulomb_matrix[m1,m2,m3,m4]) < 1.e-8:
                continue
            f.write('{} {} {} {}  {}\n'.format(m1+1,m2+1,m3+1,m4+1, \
                    coulomb_matrix[m1,m2,m3,m4]))



from __future__ import print_function
import h5py, sys, os
from pygrisb.gutz.init import initialize as ginit

def modify_gparam():
    '''convenient scipt to modify the settings in GPARAM.h5 file.
    '''
    if '-h' in sys.argv or len(sys.argv) == 1:
        print('\n inline argiment examples: \n' +
                ' -imix n -- change gimix to n \n' +
                ' -iembeddiag n -- change giembeddiag to n \n' +
                ' -dc_mode n -- change dc_mode to n \n' +
                ' -u_type n -- change u-matrix type to n \n' +
                ' -dc_nelf_list n1-n2-... -- change n_dc to n list \n' +
                ' -unique_j_ev j1-j2-... -- change unique_j_list_ev \n' +
                ' -unique_u_ev u1-u2-... -- change unique_u_list_ev \n' +
                ' -maxiter n -- change gmaxiter to n \n' +
                ' -nval_bot_ityp n1-n2-... -- change valence lower limit \n'+
                ' -nval_top_ityp n1-n2-... -- change valence upper limit')
        return
    with h5py.File('GPARAM.h5', 'a') as f:
        if '-imix' in sys.argv:
            f['/gimix'][()] = [int(sys.argv[sys.argv.index( \
                    '-imix') + 1])]
        if '-iembeddiag' in sys.argv:
            f['/giembeddiag'][()] = [int(sys.argv[sys.argv.index( \
                    '-iembeddiag') + 1])]
        if '-dc_mode' in sys.argv:
            f['/dc_mode'][()] = [int(sys.argv[sys.argv.index( \
                    '-dc_mode') + 1])]
        if '-maxiter' in sys.argv:
            f['/gmaxiter'][()] = [int(sys.argv[sys.argv.index( \
                    '-maxiter') + 1])]
        if '-dc_nelf_list' in sys.argv:
            stmp = sys.argv[sys.argv.index('-dc_nelf_list')+1]
            ne_list = [float(s) for s in stmp.split('-')]
            if '/dc_nelf_list' in f:
                f['/dc_nelf_list'][()] = ne_list
            else:
                f['/dc_nelf_list'] = ne_list
        if '-nval_bot_ityp' in sys.argv:
            stmp = sys.argv[sys.argv.index('-nval_bot_ityp')+1]
            f['/nval_bot_ityp'][()] = [int(s) for s in
                    stmp.split('-')]
        if '-nval_top_ityp' in sys.argv:
            stmp = sys.argv[sys.argv.index('-nval_top_ityp')+1]
            f['/nval_top_ityp'][()] = [int(s) for s in
                    stmp.split('-')]

    # change GMOTT.h5 if necessary
    if os.path.isfile('GMOTT.h5'):
        with h5py.File('GMOTT.h5', 'a') as f:
            if '-iembeddiag' in sys.argv:
                f['/giembeddiag'][()] = [int(sys.argv[sys.argv.index( \
                        '-iembeddiag') + 1])]
    # change settings in 'ginit.h5', re-initialize if becessary
    if '-unique_j_ev' in sys.argv or '-unique_u_ev' in sys.argv:
        re_init = True
    else:
        re_init = False

    if os.path.isfile('ginit.h5'):
        with h5py.File('ginit.h5', 'a') as f:
            if '-imix' in sys.argv:
                f['/usrqa/lnewton'][()] = int(sys.argv[sys.argv.index( \
                        '-imix') + 1])
            if '-iembeddiag' in sys.argv:
                f['/usrqa/iembeddiag'][()] = int(sys.argv[sys.argv.index( \
                        '-iembeddiag') + 1])
            if '-dc_mode' in sys.argv:
                f['/usrqa/ldc'][()] = int(sys.argv[sys.argv.index( \
                        '-dc_mode') + 1])
            if '-u_type' in sys.argv:
                f["/usrqa/u_matrix_type"][()] = int(sys.argv[sys.argv.index( \
                        '-u_type') + 1])
            if '-unique_j_ev' in sys.argv:
                stmp = sys.argv[sys.argv.index('-unique_j_ev')+1]
                f['/usrqa/unique_j_list_ev'][()] = [float(s) for s in
                        stmp.split('-')]
                f["/usrqa/u_matrix_type"][()] = [1]
            if '-unique_u_ev' in sys.argv:
                stmp = sys.argv[sys.argv.index('-unique_u_ev')+1]
                f['/usrqa/unique_u_list_ev'][()] = [float(s) for s in
                        stmp.split('-')]
                f["/usrqa/u_matrix_type"][()] = [1]
    else:
        if re_init:
            raise ValueError(' file ginit.h5 does not exist!')

    if re_init:
        ginit()



from __future__ import print_function
import os, sys, h5py, glob, pymatgen, numpy
from pygrisb.gutz.gatom import gAtoms


def read_cif():
    cif_files = glob.glob("*.cif")
    if len(cif_files) == 0:
        raise ValueError("Error: no cif file present!")
    elif len(cif_files) > 1:
        raise ValueError("Error: more than one cif files present!")
    cif_file = cif_files[0].decode('utf-8', 'ignore')
    struct = pymatgen.Structure.from_file(cif_file, primitive=True)
    struct = pymatgen.symmetry.analyzer.SpacegroupAnalyzer( \
            struct).get_primitive_standard_structure()
    from  pymatgen.io.ase import AseAtomsAdaptor
    material = AseAtomsAdaptor.get_atoms(struct)
    return material


def read_wien_structure():
    from .getwiencase import get_wiencase
    structure_file = get_wiencase() + '.struct'
    from ase.io.wien2k import read_struct
    material = read_struct(structure_file)
    return material, structure_file, None


def read_vasp_poscar():
    from ase.io.vasp import read_vasp
    material = read_vasp(filename='POSCAR')
    return material


def h5save_structure_params():
    case = None
    locrot_list = None
    if "-cif" in sys.argv:
        material = read_cif()
    elif os.path.isfile('POSCAR'):
        material = read_vasp_poscar()
    else:
        material, case, locrot_list = read_wien_structure()

    # check whether locrot_list is externally provided or not.
    if os.path.isfile('locrot.h5'):
        with h5py.File('locrot.h5', 'r') as f:
            locrot_list = f['locrot_list'][()]

    with h5py.File('ginit.h5', 'a') as f:
        symbols = material.get_chemical_symbols()
        symbols = [numpy.string_(s) for s in symbols]
        f['/struct/symbols'] = symbols
        f['/struct/cell'] = material.get_cell()
        f['/struct/scaled_positions'] = material.get_scaled_positions()
        if case is not None:
            f['/struct/case'] = case
        if locrot_list is not None:
            f['/struct/locrot_list'] = locrot_list

def get_gatoms():
    if not os.path.isfile('ginit.h5'):
        h5save_structure_params()

    with h5py.File('ginit.h5', 'r') as f:
        symbols = f['/struct/symbols'][()]
        symbols = [s.decode() for s in symbols]
        cell = f['/struct/cell'][()]
        scaled_positions = f['/struct/scaled_positions'][()]
        if '/struct/case' in f:
            case = f['/struct/case'][()]
        else:
            case = None
        if '/struct/locrot_list' in f:
            locrot_list = f['/struct/locrot_list'][()]
        else:
            locrot_list = None
    return gAtoms(symbols=symbols, cell=cell,
            scaled_positions=scaled_positions, pbc=True,
            wiencase=case, locrot_list=locrot_list)


def check_material(material, f):
    if f is None:
        return
    print(" Cell: \n", material.get_cell(), file=f)
    print(" Symbols: ", material.get_chemical_symbols(), file=f)
    print(" Scaled_positions: \n", material.get_scaled_positions(), file=f)
    print(" Mapping-table equivalent atoms: ",
            material.idx_equivalent_atoms, file=f)
    print(" Labels of correlated atoms: ", material.corr_list, file=f)
    print(" Orbital angular momentums of correlated electrons: ",
            material.df_list, file=f)
    print(" Take into accout SOC or not: ",
            material.iso, file=f)
    print(" Take into accout crystal-fields or not: ",
            material.crystal_field, file=f)
    print(" Cut-off distance for symmetry analysis: ",
            material.sym_dist_cut, file=f)
    for i, sigma in enumerate(material.sigma_list):
        print(" self energy structure ", i, file=f)
        print(sigma, file=f)




import numpy
from pygrisb.model.special import semicircular
import matplotlib.pyplot as plt


# Uniform e-mesh with different weight.
nmesh = 500
e_list = numpy.linspace(-1,1,nmesh)
sc = semicircular()
plt.plot(e_list, sc.dos(e_list), label='dos')
plt.plot(e_list, sc.cdos(e_list), label='cdos')
plt.xlim(-1.0, 1.0)
plt.ylim(0, 1.0)
plt.legend()
plt.xlabel('$\omega$')
plt.savefig('semicir_dos.png')



"""
Given the matrix structure, generate the (Hermitian) matrix basis set.
"""
import numpy as np
from scipy.sparse import csc_matrix


def sigmatomatrixbasis(sigma):
    '''
    Generate Hermitian matrix basis set.
    '''
    matrix_basis = []
    sigma = np.asarray(sigma)
    for element in range(np.max(sigma), 0, -1):
        spots = np.argwhere(sigma == element)
        num_spots = len(spots)
        if num_spots == 0:
            continue
        spots = spots.T
        # Skip if located at lower trigonal block
        if spots[0][0] > spots[1][0]:
            continue
        if spots[0][0] == spots[1][0]:
            value = 1 / np.sqrt(float(num_spots))
            matrix = np.zeros_like(sigma, dtype=np.complex)
            matrix[spots[0], spots[1]] = value
            matrix_basis.append(matrix)
        else:
            # non-zero element
            value = 1 / np.sqrt(float(num_spots * 2))
            matrix = np.zeros_like(sigma, dtype=np.complex)
            matrix[spots[0], spots[1]] = value
            matrix[spots[1], spots[0]] = value
            matrix_basis.append(matrix)
            value = value * 1.j
            matrix = np.zeros_like(sigma, dtype=np.complex)
            matrix[spots[0], spots[1]] = value
            matrix[spots[1], spots[0]] = -value
            matrix_basis.append(matrix)
    return matrix_basis


def listsigmatomatrixbasis(sigma_list):
    matrix_basis_list = []
    for sigma in sigma_list:
        matrix_basis_list.append(SigmaToMatrixBasis(sigma))
    return matrix_basis_list


def matrixstructtobasis(m_struct):
    '''
    Generate general non-Hermitian matrix basis set.
    '''
    matrix_basis = []
    m_struct = np.asarray(m_struct)
    for element in range(np.max(m_struct), 0, -1):
        spots = np.argwhere(m_struct == element)
        num_spots = len(spots)
        if num_spots == 0:
            continue
        spots = spots.T
        value = 1 / np.sqrt(float(num_spots))
        matrix = np.zeros_like(m_struct, dtype=np.complex)
        matrix[spots[0], spots[1]] = value
        matrix_basis.append(matrix)
    return matrix_basis


def listmatrixstructtobasis(m_struct_list):
    matrix_basis_list = []
    for m_struct in m_struct_list:
        matrix_basis_list.append(MatrixStructToBasis(m_struct))
    return matrix_basis_list


def hermitian_csc_matrix(i, j, n):
    if i == j:
        return csc_matrix(([1.0], [[i], [j]]), \
                shape=(n, n), dtype=complex)
    else:
        x = 1.j/np.sqrt(2.)
        return csc_matrix(([x, -x], [[i, j], [j, i]]), \
                shape=(n, n), dtype=complex)


def hermitian_csc_matrix_basis(n, istart=0):
    '''
    Generate hermitian matrix basis set of dimention n.
    '''
    return [hermitian_csc_matrix(i, j, n) for i in range(istart, n) \
            for j in range(i, n)]
